package org.jeecg.modules.adweb.enquiry.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.xkcoding.http.util.StringUtil;
import jakarta.annotation.PostConstruct;
import jakarta.annotation.Resource;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import java.util.stream.Stream;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.jeecg.common.constant.CacheConstant;
import org.jeecg.common.constant.WebsocketConst;
import org.jeecg.common.system.vo.DictModel;
import org.jeecg.common.system.vo.DictPropertyModel;
import org.jeecg.common.util.FastJsonUtil;
import org.jeecg.modules.adweb.common.constant.NumConstant;
import org.jeecg.modules.adweb.common.dto.CountryAreaApiDto;
import org.jeecg.modules.adweb.common.jedis.ObjectUtil;
import org.jeecg.modules.adweb.common.service.FeishuService;
import org.jeecg.modules.adweb.common.service.WebSocketService;
import org.jeecg.modules.adweb.common.util.*;
import org.jeecg.modules.adweb.enquiry.constant.EnquiryConstants;
import org.jeecg.modules.adweb.enquiry.constant.EnquiryDistributeTypeConstant;
import org.jeecg.modules.adweb.enquiry.constant.EnquirySendStatus;
import org.jeecg.modules.adweb.enquiry.dto.EnquiryDTO;
import org.jeecg.modules.adweb.enquiry.dto.param.EffectiveEnquiryParamDto;
import org.jeecg.modules.adweb.enquiry.dto.result.EnquirySpecialFields;
import org.jeecg.modules.adweb.enquiry.entity.*;
import org.jeecg.modules.adweb.enquiry.mapper.AdwebEnquiryMapper;
import org.jeecg.modules.adweb.enquiry.service.*;
import org.jeecg.modules.adweb.site.entity.AdwebSite;
import org.jeecg.modules.adweb.site.service.IAdwebSiteService;
import org.jeecg.modules.adweb.site.service.ISiteUserPermissionService;
import org.jeecg.modules.adweb.subscription.mapper.UserPlanSubscriptionMapper;
import org.jeecg.modules.adweb.system.entity.SysException;
import org.jeecg.modules.adweb.system.service.IMasterSubAccountRelationService;
import org.jeecg.modules.adweb.system.service.ISysExceptionService;
import org.jeecg.modules.adweb.system.service.impl.SysAdwebApiImpl;
import org.jeecg.modules.adweb.userCountry.entity.AdwebCountry;
import org.jeecg.modules.adweb.userCountry.entity.AdwebUserCountry;
import org.jeecg.modules.adweb.userCountry.service.IAdwebCountryService;
import org.jeecg.modules.adweb.userCountry.service.IAdwebUserCountryService;
import org.jeecg.modules.adweb.workflow.service.IExecuteNodeService;
import org.jeecg.modules.message.websocket.WebSocket;
import org.jeecg.modules.system.entity.SysDictItem;
import org.jeecg.modules.system.entity.SysUser;
import org.jeecg.modules.system.service.ISysDictService;
import org.jeecg.modules.system.service.ISysUserService;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Lazy;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

/**
 * @Description: 询盘信息存储表单 @Author: jeecg-boot @Date: 2024-10-12 @Version: V1.0
 */
@Slf4j
@Service
public class AdwebEnquiryServiceImpl extends ServiceImpl<AdwebEnquiryMapper, AdwebEnquiry>
    implements IAdwebEnquiryService {

  @Resource private AdwebEnquiryMapper adwebEnquiryMapper;

  @Resource @Lazy private ISysUserService sysUserService;

  @Resource private IMasterSubAccountRelationService masterSubAccountRelationService;

  @Resource private GeoIpUtil geoIpUtil;

  @Resource private IAdwebPublicEnquiryRuleService AdwebPublicEnquiryRuleService;

  @Resource private ISysDictService dictService;

  @Resource private SnowflakeIdUtil snowflakeIdUtil;

  @Resource private AdwebRedisUtil adwebRedisUtil;

  @Resource private ISysExceptionService sysExceptionService;

  @Resource private FeishuService feishuService;

  @Resource private IAdwebEnquiryBlacklistService adwebEnquiryBlacklistService;

  @Resource private IAdwebSiteBlackEmailService enquirySiteBlackEmailService;

  @Resource private IAdwebPublicBlackEmailService adwebPublicBlackEmailService;

  @Resource private IAdwebPublicBlackIpService adwebPublicBlackIpService;

  @Resource private IAdwebSiteBlackIpService adwebSiteBlackIpService;

  @Resource private IAdwebSiteEnquiryRuleService adwebSiteEnquiryRuleService;

  @Resource private ISiteUserPermissionService siteUserPermissionService;

  @Resource private IEnquiryEmailMessageService enquiryEmailMessageService;

  @Resource private IAdwebSiteService adwebSiteService;

  @Resource private IAdwebCountryService adwebCountryService;

  @Resource private IAdwebUserCountryService adwebUserCountryService;

  @Resource private AdwebOpenApiService adwebOpenApiService;

  @Resource private IAdwebEnquiryFormService adwebEnquiryFormService;

  @Resource private SysAdwebApiImpl sysAdwebApiImpl;

  @Resource private WebSocket webSocket;

  @Resource private WebSocketService webSocketService;

  @Resource private EnquiryRedisService enquiryRedisService;

  @Resource private ISubUserEmailService subUserEmailService;

  @Resource private UserPlanSubscriptionMapper userPlanSubscriptionMapper;

  @Resource private IExecuteNodeService executeNodeService;

  private static final byte[] redisKey = EnquiryConstants.ENQUIRY_EMAIL.getBytes();

  private static final byte[] siteRedisKey = EnquiryConstants.ENQUIRY_SITE.getBytes();

  // 询盘唯一键 redis key
  public static final String EnquiryUniqueKey = "ENQUIRY_UNIQUE_KEY::";

  // Email Redis key
  private static final String EmailTenMinKey = "JUDGE_WASTE_ENQUIRY_EMAIL_BY_10_MIN::";
  private static final String EmailOneDayKey = "JUDGE_WASTE_ENQUIRY_EMAIL_BY_ONE_DAY::";
  private static final String BlackEmailKey = "BLACK_EMAIL_LIST";
  private static final String WhiteEmailListKey = "WHITE_EMAIL_LIST";
  private static final String NotBlackEmailWasteEnquiryKey = "NOT_BLACK_EMAIL_WASTE_ENQUIRY_MAP::";

  // site Email Redis key
  private static final String SiteEmailTenMinKey = "JUDGE_WASTE_SITE_ENQUIRY_EMAIL_BY_10_MIN::";
  private static final String SiteEmailOneDayKey = "JUDGE_WASTE_SITE_ENQUIRY_EMAIL_BY_ONE_DAY::";
  private static final String SiteBlackEmailKey = "SITE_BLACK_EMAIL_LIST";
  private static final String SiteWhiteEmailListKey = "SITE_WHITE_EMAIL_LIST";
  private static final String NotBlackEmailWasteSiteEnquiryKey =
      "NOT_BLACK_EMAIL_WASTE_SITE_ENQUIRY_MAP::";

  // ip Reids Key
  private static final String IpTenMinKey = "JUDGE_WASTE_ENQUIRY_IP_BY_10_MIN::";
  private static final String IpOneDayKey = "JUDGE_WASTE_ENQUIRY_IP_BY_ONE_DAY::";
  private static final String BlackIpKey = "BLACK_IP_LIST";
  private static final String WhiteIpListKey = "WHITE_IP_LIST";
  private static final String NotBlackIpWasteEnquiryKey = "NOT_BLACK_IP_WASTE_ENQUIRY_MAP::";

  // site ip Reids Key
  private static final String SiteIpTenMinKey = "JUDGE_WASTE_ENQUIRY_SITE_IP_BY_10_MIN::";
  private static final String SiteIpOneDayKey = "JUDGE_WASTE_ENQUIRY_SITE_IP_BY_ONE_DAY::";
  private static final String NotSiteBlackIpWasteEnquiryKey =
      "NOT_SITE_BLACK_IP_WASTE_ENQUIRY_MAP::";
  private static final String SiteBlackIpKey = "SITE_BLACK_IP_LIST";
  private static final String SiteWhiteIpListKey = "SITE_WHITE_IP_LIST";

  // 垃圾询盘原因
  private static final String EMPTY_MESSAGE_REASON = "系统操作-通过询盘内容为空检测到垃圾询盘";
  private static final String EMAIL_BLACKLIST_REASON = "系统操作-通过公共和站点的询盘邮箱黑名单检测到垃圾询盘-%s";
  private static final String EMAIL_PUBLIC_BLACKLIST_REASON = "系统操作-通过公共的询盘邮箱黑名单检测到垃圾询盘-%s";
  private static final String EMAIL_SITE_BLACKLIST_REASON = "系统操作-通过站点的询盘邮箱黑名单检测到垃圾询盘-%s";
  private static final String IP_BLACKLIST_REASON = "系统操作-通过询盘公共和站点ip黑名单检测到垃圾询盘-%s";
  private static final String IP_PUBLIC_BLACKLIST_REASON = "系统操作-通过询盘公共ip黑名单检测到垃圾询盘-%s";
  private static final String IP_SITE_BLACKLIST_REASON = "系统操作-通过询盘站点ip黑名单检测到垃圾询盘-%s";
  private static final String URL_BLACKLIST_REASON = "系统操作-通过站点url检测到垃圾询盘-%s";
  private static final String KEYWORD_BLACKLIST_REASON = "系统操作-通过公共和站点关键词黑名单检测到垃圾询盘-%s";
  private static final String KEYWORD_PUBLIC_BLACKLIST_REASON = "系统操作-通过公共关键词黑名单检测到垃圾询盘-%s";
  private static final String KEYWORD_SITE_BLACKLIST_REASON = "系统操作-通过站点关键词黑名单检测到垃圾询盘-%s";
  private static final String LONG_WORD_REASON = "系统操作-通过单词长度超过50个字符检测到垃圾询盘";

  @Value("${judge_waste_enquiry.email.tenMinNum}")
  private Integer emailTenMinNum;

  @Value("${judge_waste_enquiry.email.oneDayNum}")
  private Integer emailOneDayNum;

  @Value("${judge_waste_enquiry.ip.tenMinNum}")
  private Integer ipTenMinNum;

  @Value("${judge_waste_enquiry.ip.oneDayNum}")
  private Integer ipOneDayNum;

  @Value("${judge_waste_enquiry.email.notBlackListNum}")
  private Integer emailNotBlackListNum;

  @Value("${judge_waste_enquiry.ip.notBlackListNum}")
  private Integer ipNotBlackListNum;

  @Value("${judge_waste_enquiry.email.notBlackListDate}")
  private Integer emailNotBlackListDate;

  @Value("${judge_waste_enquiry.ip.notBlackListDate}")
  private Integer ipNotBlackListDate;

  private static final int USER_EFFECTIVE_ACTIVE = 1;
  private static final int USER_EFFECTIVE_WAIT_ACTIVE = 2;

  @PostConstruct
  public void startEnquiryRedisListener() {
    enquiryRedisService.init();
    enquiryRedisService.startEnquiryEmailMessageListener(
        EnquiryConstants.ENQUIRY_EMAIL_CHANNEL, enquiryEmailMessageService);
  }

  // 获取有效公共询盘规则
  private List<String> getPublicEnquiryRules(int blackOrWhite, int useStatus) {

    List<String> blackWordList = null;
    try {
      List<AdwebPublicEnquiryRule> AdwebPublicEnquiryRuleList =
          AdwebPublicEnquiryRuleService.list(
              new LambdaQueryWrapper<AdwebPublicEnquiryRule>()
                  .eq(AdwebPublicEnquiryRule::getStatus, 1)
                  .eq(AdwebPublicEnquiryRule::getIsEnable, 1)
                  .eq(AdwebPublicEnquiryRule::getBlackOrWhiteList, blackOrWhite)
                  .eq(AdwebPublicEnquiryRule::getUseStatus, useStatus));
      if (CollectionUtils.isNotEmpty(AdwebPublicEnquiryRuleList)) {
        blackWordList =
            AdwebPublicEnquiryRuleList.stream().map(AdwebPublicEnquiryRule::getWord).toList();
      }
    } catch (Exception e) {
      log.error("读取询盘黑名单关键词失败", e);
    }

    return blackWordList;
  }

  /** 统计管理的询盘数量 */
  @Override
  public Map<String, Integer> getEnquiryCount(List<String> uids) {
    Map<String, Integer> map = new HashMap<>();
    List<JSONObject> list = adwebEnquiryMapper.getEnquiryCount(uids);
    if (CollectionUtils.isNotEmpty(list))
      for (JSONObject object : list) {
        map.put(object.getString("uid"), object.getInteger("enquiryCount"));
      }
    return map;
  }

  /** 统计管理的询盘数量 */
  @Override
  public int getEnquiryCount(String uid) {
    List<String> codeList = siteUserPermissionService.getSiteCodeList(uid);
    QueryWrapper<AdwebEnquiry> queryWrapper = new QueryWrapper<>();
    queryWrapper.eq("status", 1);
    queryWrapper.in("site_code", codeList);
    return (int) this.count(queryWrapper);
  }

  /**
   * 添加询盘
   *
   * @param enquiryDto 询盘对象
   * @return
   */
  public void addEnquiry(EnquiryDTO enquiryDto, String plugin, AdwebSite adwebSite) {
    log.info("=============开始进行询盘处理==============：{}", enquiryDto.getSiteHost());
    // 获取公共黑名单关键词
    List<String> blackWordList = getPublicEnquiryRules(0, 0);

    // 获取子账户
    Map<String, String> principalEmailMap = new HashMap<>();
    List<String> subAccountIdList =
        masterSubAccountRelationService.getSubAccountIdByMaster(String.valueOf(adwebSite.getUid()));
    log.info("获取的子账户id：{}", FastJsonUtil.toJSONString(subAccountIdList));
    if (CollectionUtils.isNotEmpty(subAccountIdList)) {
      subAccountIdList.add(String.valueOf(adwebSite.getUid()));
      List<SysUser> principalEmailList =
          sysUserService.list(
              new LambdaQueryWrapper<SysUser>()
                  .in(SysUser::getId, subAccountIdList)
                  .eq(SysUser::getDelFlag, 0)
                  .isNotNull(SysUser::getEmail)
                  .ne(SysUser::getEmail, "")
                  .select(SysUser::getId, SysUser::getEmail));
      if (CollectionUtils.isNotEmpty(principalEmailList)) {
        principalEmailMap =
            principalEmailList.stream()
                .collect(Collectors.toMap(SysUser::getId, SysUser::getEmail));
      }
    }

    List<AdwebEnquiry> adwebEnquiryList = new ArrayList<>();
    AdwebEnquiryForm form = new AdwebEnquiryForm();

    try {
      AdwebEnquiry adwebEnquiry = createEnquiry(enquiryDto, adwebSite);

      String forms = FastJsonUtil.toJSONString(enquiryDto.getForms());
      parseFormAndSetFields(adwebEnquiry, forms, adwebSite);

      if (StringUtils.isNotBlank(adwebEnquiry.getFromIp())) {
        setCountryByIp(adwebEnquiry);
      }

      setPrincipal(adwebEnquiry, adwebSite.getEnquiryDistributeType(), adwebSite.getId());
      adwebEnquiryList.add(adwebEnquiry);

      form.setEnquiryId(adwebEnquiry.getId());
      form.setForm(forms);
      form.setCountry(adwebEnquiry.getCountry());
      form.setPluginName(plugin);

    } catch (Exception e) {
      log.error("消费站点为：{}， recordId为：{} 的询盘出现问题", adwebSite.getName(), enquiryDto.getRecordId(), e);
    }

    // 公共黑名单，站点黑名单过滤
    judgeWasteEnquiryBySiteId(adwebEnquiryList, adwebSite, blackWordList);

    // 客户黑名单过滤
    judgeBlacklist(adwebEnquiryList);

    AdwebEnquiry target = adwebEnquiryList.get(0);

    // 管理控制后台添加假询盘时，不做唯一性条件录入
    if (!"adweb3".equalsIgnoreCase(plugin)) {
      // 获取最近一周的询盘数量，根据域名和record id 生成唯一标识，存入redis，用来过滤之后的询盘mq，防止重复消费
      // 记录开始时间
      long startTime = System.currentTimeMillis();
      String EnquiryRedisUniqueKey = EnquiryUniqueKey + target.getSiteHost();

      if (adwebRedisUtil.hasKey(EnquiryRedisUniqueKey)) {
        List<String> enquiryUniqueIds = getListFromRedis(EnquiryRedisUniqueKey);

        // 防止redis 唯一键存储过大，并始终更新
        enquiryUniqueIds.remove(enquiryUniqueIds.size() - 1); // 移除第一个唯一键
        enquiryUniqueIds.add(target.getSiteHost() + target.getRecordId()); // 添加最新的唯一键

        adwebRedisUtil.set(EnquiryRedisUniqueKey, enquiryUniqueIds, 60 * 60 * 24);

      } else {
        Date start = DateUtil.addDays(new Date(), -7);
        List<EnquiryDTO> enquiryList =
            this.enquiryList(
                target.getSiteCode(),
                DateUtil.formatDate(start, DateUtil.DATE_TIME_FORMAT),
                DateUtil.formatDate(new Date(), DateUtil.DATE_TIME_FORMAT));

        if (enquiryList != null) {
          List<String> enquiryUniqueIds =
              enquiryList.stream()
                  .map(enquiry -> enquiry.getSiteHost() + enquiry.getRecordId())
                  .collect(Collectors.toList());

          adwebRedisUtil.set(EnquiryRedisUniqueKey, enquiryUniqueIds, 60 * 60 * 24);
        }
      }
      // 记录结束时间
      long endTime = System.currentTimeMillis();
      log.info(
          "站点：{}，询盘ID：{} 唯一性条件处理耗时：{} ms",
          enquiryDto.getSiteHost(),
          enquiryDto.getRecordId(),
          endTime - startTime);
    }

    try {
      this.save(target);
      adwebEnquiryFormService.save(form);

    } catch (Exception e) {
      log.error(
          "站点为：{}，询盘ID：{} 保存询盘到数据库失败，原因是:{}",
          adwebSite.getName(),
          enquiryDto.getRecordId(),
          e.getMessage());
    }

    // 记录开始时间
    long startTimeWBS = System.currentTimeMillis();
    // 站内websocket通知
    AdwebSite targetSite = adwebSiteService.getSiteByCode(target.getSiteCode());
    SysUser sysUser = sysUserService.getUserByName("admin");
    String[] userIds = {targetSite.getUid(), sysUser.getId()};
    webSocketService.sendMessage(userIds, "询盘通知", "您已经收到询盘，请尽快处理", WebsocketConst.CMD_ENQUIRY);
    // 记录开始时间
    long endTimeWBS = System.currentTimeMillis();

    log.info(
        "站点：{}，询盘ID：{} 处理websocket消息耗时：{} ms",
        enquiryDto.getSiteHost(),
        enquiryDto.getRecordId(),
        endTimeWBS - startTimeWBS);

    // 发送询盘邮件
    sendValidateEmail(target, adwebSite, principalEmailMap);

    // 如果站点套餐是苏豪全托管，则创建有效询盘审核流程
    Map<String, String> subscribePlan =
        userPlanSubscriptionMapper.getSubscribePlanBySiteId(adwebSite.getId());

    if ("苏豪通全托管".equals(subscribePlan.get("planName")) && target.getUserEffective() == 1) {
      executeNodeService.copyEnquiryVerifyFlow(target.getId().toString());
    }
  }

  /** 设置询盘负责人 */
  private void setPrincipal(AdwebEnquiry adwebEnquiry, String distributeType, Integer siteId) {

    // 根据询盘发送规则
    if (EnquiryDistributeTypeConstant.MASTER.equals(distributeType)) {
      adwebEnquiry.setPrincipalUid(adwebEnquiry.getUid());
    } else if (EnquiryDistributeTypeConstant.ALL.equals(distributeType)) {
      adwebEnquiry.setPrincipalUid("ALL");
    } else if (EnquiryDistributeTypeConstant.REGION.equals(distributeType)) {
      String countryCode = adwebEnquiry.getCountryCode();
      if (StringUtils.isNotBlank(countryCode)) {
        LambdaQueryWrapper<AdwebCountry> countryQueryWrapper =
            new LambdaQueryWrapper<AdwebCountry>().eq(AdwebCountry::getCountryIsoCode, countryCode);
        AdwebCountry country = adwebCountryService.getOne(countryQueryWrapper);
        if (country != null) {
          String countryId = country.getGeonameId();
          QueryWrapper<AdwebUserCountry> userCountryQueryWrapper = new QueryWrapper<>();
          userCountryQueryWrapper.eq("status", 1);
          userCountryQueryWrapper.eq("site_id", siteId);
          userCountryQueryWrapper.eq("country_id", countryId);
          AdwebUserCountry adwebUserCountry =
              adwebUserCountryService.getOne(userCountryQueryWrapper);
          if (adwebUserCountry != null && StringUtils.isNotBlank(adwebUserCountry.getUid())) {
            adwebEnquiry.setPrincipalUid(adwebUserCountry.getUid());
          } else {
            adwebEnquiry.setPrincipalUid(adwebEnquiry.getUid());
          }
        }
      }
    }
    adwebEnquiry.setPrincipalType(0);

    // 数据字典中配置某些网站只发给主账户
    List<DictModel> contactEnquiryWebsites =
        dictService.queryDictItemsByCode("send_contact_enquiry_website");
    List<String> contactEnquiryCodes =
        contactEnquiryWebsites.stream().map(DictModel::getValue).collect(Collectors.toList());
    if (contactEnquiryCodes.contains(adwebEnquiry.getSiteCode())) {
      if (StringUtils.isNotBlank(adwebEnquiry.getFromPage())
          && adwebEnquiry.getFromPage().contains("contact")) {
        adwebEnquiry.setPrincipalType(1);
        adwebEnquiry.setExistContactEnquiry(1);
      }
    }
  }

  /**
   * 发送询盘邮件
   *
   * @param adwebEnquiry 待发送询盘列表
   * @param adwebSite 接收邮箱集合
   * @param principalEmailMap
   */
  private void sendValidateEmail(
      AdwebEnquiry adwebEnquiry, AdwebSite adwebSite, Map<String, String> principalEmailMap) {
    log.info("需要发送邮件的询盘：{}", FastJsonUtil.toJSONString(adwebEnquiry.getId()));

    // 特殊站点发送邮件
    List<DictPropertyModel> siteRes =
        sysAdwebApiImpl.queryDictInfoByDictCode("turn_inquiry_site_code");
    if (CollectionUtils.isNotEmpty(siteRes)
        && siteRes.get(0).getValue().equals(adwebSite.getCode())) {
      log.info("该询盘：{}， 是特殊站点发送邮件", adwebEnquiry.getId());
      specialSendEmail(adwebEnquiry, adwebSite);
    } else {
      String emailStr = adwebSite.getEnquiryEmailList();
      if (StringUtil.isEmpty(emailStr)) {
        log.info("该站点未获取到站点配置邮箱：{}", adwebSite.getName());
      }
      log.info("发送询盘邮件--站点邮箱：{}", emailStr);
      log.info("发送询盘邮件--子账户邮箱：{}", FastJsonUtil.toJSONString(principalEmailMap));

      List<String> emailList = new ArrayList<>();
      if (StringUtils.isNotBlank(emailStr)) {
        emailList = JSON.parseArray(emailStr, String.class);
      }

      SysUser sysUser = sysUserService.getById(adwebEnquiry.getUid());

      if (sysUser != null && !emailList.contains(sysUser.getEmail())) {
        emailList.add(sysUser.getEmail());
      }

      log.info("发送询盘邮件--站点包含用户邮箱：{}", FastJsonUtil.toJSONString(emailList));

      if (CollectionUtils.isEmpty(emailList)) {
        log.error("该站点未配置主账号邮箱和站点邮箱");
        return;
      }

      List<EnquiryEmailMessage> enquiryEmailMessageList = new ArrayList<>();

      if (adwebEnquiry.getWasteEnquiry() != 1 && adwebEnquiry.getStatus() != 0) {
        HashSet<String> emailSet = new HashSet<>();
        String priUid = adwebEnquiry.getPrincipalUid();

        if (principalEmailMap.isEmpty()) {
          for (String e : emailList) {
            if (emailSet.contains(e)) {
              continue;
            }
            enquiryEmailMessageList.add(newEmail(adwebEnquiry, e));
            emailSet.add(e);
          }
        }
        // 1 如果选择发送全部 2.不是产品页面 全发
        else if (StringUtil.isEmpty(adwebEnquiry.getPrincipalUid())
            || adwebEnquiry.getExistProductEnquiry() == null
            || adwebEnquiry.getExistProductEnquiry() == 0
            || adwebEnquiry.getPrincipalUid().equals("ALL")) {
          if (adwebSite.getEnquirySendEmailType().equals("ALL")) {
            log.info("发送询盘邮件--不是产品--进行所有账号发送规则");
            for (String e : principalEmailMap.values()) {
              if (emailSet.contains(e)) {
                continue;
              }
              enquiryEmailMessageList.add(newEmail(adwebEnquiry, e));
              emailSet.add(e);
            }
          } else {
            // 询盘邮件规则 主账号和添加了的负责人,如果负责人选择了all取所有人
            log.info("发送询盘邮件--不是产品--进行只有主账号发送规则--负责人不为all");
            if (!"ALL".equals(adwebEnquiry.getPrincipalUid())) {
              // 主账号
              SysUser masterUserAccount = sysUserService.getById(adwebEnquiry.getUid());
              if (masterUserAccount != null
                  && StringUtils.isNotBlank(masterUserAccount.getEmail())) {
                emailSet.add(masterUserAccount.getEmail());
                enquiryEmailMessageList.add(newEmail(adwebEnquiry, masterUserAccount.getEmail()));
              }
              // 负责人
              for (String key : principalEmailMap.keySet()) {
                if (adwebEnquiry.getPrincipalUid().equals(key)
                    && !emailSet.contains(principalEmailMap.get(key))) {
                  enquiryEmailMessageList.add(newEmail(adwebEnquiry, principalEmailMap.get(key)));
                  emailSet.add(principalEmailMap.get(key));
                }
              }
            } else {
              // 判断是否走同步询盘，同步询盘不进行全部处理
              log.info("发送询盘邮件--不是产品--进行只有主账号发送规则--负责人为all");
              if (StringUtil.isEmpty(adwebEnquiry.getIsEnquirySync())) {
                for (String e : principalEmailMap.values()) {
                  if (emailSet.contains(e)) {
                    continue;
                  }
                  enquiryEmailMessageList.add(newEmail(adwebEnquiry, e));
                  emailSet.add(e);
                }
              } else {
                // 主账号
                SysUser masterUserAccount = sysUserService.getById(adwebEnquiry.getUid());
                if (masterUserAccount != null
                    && StringUtils.isNotBlank(masterUserAccount.getEmail())) {
                  emailSet.add(masterUserAccount.getEmail());
                  enquiryEmailMessageList.add(newEmail(adwebEnquiry, masterUserAccount.getEmail()));
                }
              }
            }
          }
          for (String e : emailList) {
            if (emailSet.contains(e)) {
              continue;
            }
            enquiryEmailMessageList.add(newEmail(adwebEnquiry, e));
            emailSet.add(e);
          }
        }
        // 当主站为空
        else if (StringUtil.isEmpty(principalEmailMap.get(adwebEnquiry.getUid()))
            && adwebEnquiry.getExistProductEnquiry() == 1) {
          log.info("发送询盘邮件--产品询盘--当主站为空");
          String email = principalEmailMap.get(priUid);
          List<String> subSiteEmailList = null;
          if (StringUtils.isNotBlank(email)) {
            enquiryEmailMessageList.add(newEmail(adwebEnquiry, email));
            emailSet.add(email);
            // 筛选出子站点，不给子站点发，给配置询盘的地方发
            subSiteEmailList =
                principalEmailMap.entrySet().stream()
                    .filter(entry -> !email.equals(entry.getKey()))
                    .map(Map.Entry::getValue)
                    .collect(Collectors.toList());
          } else {
            subSiteEmailList = new ArrayList<>(principalEmailMap.values());
          }

          // 遍历emailList 不包含subSiteEmailList
          for (String e : emailList) {
            if (subSiteEmailList.contains(e) || emailSet.contains(e)) {
              continue;
            }
            enquiryEmailMessageList.add(newEmail(adwebEnquiry, e));
            emailSet.add(e);
          }
        }
        // 如果当前询盘配置的为子账户 则给子账户和主账户发
        else if (adwebEnquiry.getExistProductEnquiry() == 1
            && !adwebEnquiry.getUid().equals(priUid)) {
          log.info("发送询盘邮件--产品询盘--如果当前询盘配置的为子账户 则给子账户和主账户发");
          String email = principalEmailMap.get(priUid);
          if (StringUtils.isNotBlank(email)) {
            enquiryEmailMessageList.add(newEmail(adwebEnquiry, email));
            emailSet.add(email);
          }

          // 给主账户发
          String masterEmail = principalEmailMap.get(adwebEnquiry.getUid());

          enquiryEmailMessageList.add(newEmail(adwebEnquiry, masterEmail));
          emailSet.add(masterEmail);

          // 筛选出子站点，不给子站点发，给配置询盘的地方发
          List<String> subSiteEmailList =
              principalEmailMap.entrySet().stream()
                  .filter(entry -> !Arrays.asList(email, masterEmail).contains(entry.getKey()))
                  .map(Map.Entry::getValue)
                  .toList();

          // 遍历emailList 不包含subSiteEmailList
          for (String e : emailList) {
            if (subSiteEmailList.contains(e) || emailSet.contains(e)) {
              continue;
            }
            enquiryEmailMessageList.add(newEmail(adwebEnquiry, e));
            emailSet.add(e);
          }
        }
        // 如果当前询盘配置的为主账户 则只给主账户发 //还有一种情况 子账户为空 只给主站发
        else if (adwebEnquiry.getUid().equals(priUid)
            && adwebEnquiry.getExistProductEnquiry() == 1) {
          log.info("发送询盘邮件--产品询盘--如果当前询盘配置的为主账户 则只给主账户发 //还有一种情况 子账户为空 只给主站发");
          String email = principalEmailMap.get(priUid);
          enquiryEmailMessageList.add(newEmail(adwebEnquiry, email));
          emailSet.add(email);

          // 筛选出子站点，不给子站点发，给配置询盘的地方发
          List<String> subSiteEmailList =
              principalEmailMap.entrySet().stream()
                  .filter(entry -> !entry.getKey().equals(priUid))
                  .map(Map.Entry::getValue)
                  .toList();

          // 遍历emailList 不包含subSiteEmailList
          for (String e : emailList) {
            if (subSiteEmailList.contains(e) || emailSet.contains(e)) {
              continue;
            }
            enquiryEmailMessageList.add(newEmail(adwebEnquiry, e));
            emailSet.add(e);
          }
        }

        log.info("=======发送询盘邮件======");
        enquiryEmailMessageList.forEach(e -> System.out.println(e.getEmail()));
        log.info("需要添加的询盘邮件：{}", FastJsonUtil.toJSONString(enquiryEmailMessageList));
        if (CollectionUtils.isNotEmpty(enquiryEmailMessageList)) {
          LambdaQueryWrapper<EnquiryEmailMessage> enquiryEmailMessageQueryWrapper =
              new LambdaQueryWrapper<>();
          enquiryEmailMessageQueryWrapper.eq(EnquiryEmailMessage::getSiteCode, adwebSite.getCode());
          List<EnquiryEmailMessage> enquiryEmailMessages =
              enquiryEmailMessageService.list(enquiryEmailMessageQueryWrapper);
          List<EnquiryEmailMessage> sameEnquiryEmailMessageList = new ArrayList<>();
          if (CollectionUtils.isNotEmpty(enquiryEmailMessages)) {
            for (EnquiryEmailMessage enquiryEmailMessage : enquiryEmailMessages) {
              for (EnquiryEmailMessage enquiryEmailMessage1 : enquiryEmailMessageList) {
                if (enquiryEmailMessage.getEnquiryId().equals(enquiryEmailMessage1.getEnquiryId())
                    && enquiryEmailMessage.getEmail().equals(enquiryEmailMessage1.getEmail())) {
                  sameEnquiryEmailMessageList.add(enquiryEmailMessage1);
                }
              }
            }
            enquiryEmailMessageList.removeAll(sameEnquiryEmailMessageList);
          }
          log.info("最终添加的询盘邮件：{}", FastJsonUtil.toJSONString(enquiryEmailMessageList));
          enquiryEmailMessageService.saveBatch(enquiryEmailMessageList);
          try {
            for (EnquiryEmailMessage message : enquiryEmailMessageList) {
              JedisUtil.lpush(redisKey, ObjectUtil.object2Bytes(message.getEnquiryId()));
              JedisUtil.publishMsg(
                  EnquiryConstants.ENQUIRY_EMAIL_CHANNEL, EnquiryConstants.ENQUIRY_EMAIL);
            }
          } catch (Exception e) {
            log.info("推送发送询盘邮件的消息失败", e);
          }
        }
      } else {
        log.info("=======该垃圾询盘不发送询盘邮件======{}", adwebEnquiry.getId());
      }
    }
  }

  /**
   * 特殊账号发送邮件
   *
   * @param adwebEnquiry
   * @param adwebSite
   */
  public void specialSendEmail(AdwebEnquiry adwebEnquiry, AdwebSite adwebSite) {
    // 通过主账号id获取子账号id集合
    List<String> idList =
        masterSubAccountRelationService.getSubAccountIdByMaster(adwebSite.getUid());

    // 获取子账号信息
    // TODO chenpeiqing: 字段在V3中不存在
    QueryWrapper<SysUser> sysUserQueryWrapper = new QueryWrapper<>();
    sysUserQueryWrapper.eq("is_inquiry", 2);
    sysUserQueryWrapper.eq("status", 1);
    if (CollectionUtils.isNotEmpty(idList)) {
      sysUserQueryWrapper.in("id", idList);
    }
    List<SysUser> sysUsers = sysUserService.list(sysUserQueryWrapper);

    List<EnquiryEmailMessage> enquiryEmailMessageList = new ArrayList<>();
    if (adwebEnquiry.getWasteEnquiry() != 1 && adwebEnquiry.getStatus() != 0) {
      // 当前发送邮件的账号
      SysUser sysUser = new SysUser();

      // 如果没有子账号，则只给主账号发送邮件
      if (CollectionUtils.isEmpty(sysUsers)) {
        // 获取主账号
        SysUser mainUser = sysUserService.getById(adwebSite.getUid());
        // 主账号邮件信息添加
        if (mainUser != null && StringUtils.isNotBlank(mainUser.getEmail())) {
          enquiryEmailMessageList.add(newEmail(adwebEnquiry, mainUser.getEmail()));
        }
      } else {
        // 轮流发送邮件逻辑处理
        List<SubUserEmail> list = subUserEmailService.list();
        if (CollectionUtils.isEmpty(list)) {
          sysUser = sysUsers.get(0);
        } else {
          SubUserEmail subUserEmail = list.get(0);
          subUserEmailService.removeAll();

          SysUser user = sysUserService.getById(subUserEmail.getId());
          int index = sysUsers.indexOf(user);
          if (index == -1 || index == sysUsers.size() - 1) {
            sysUser = sysUsers.get(0);
          } else {
            sysUser = sysUsers.get(index + 1);
          }
        }
        SubUserEmail subUserEmail = new SubUserEmail();
        subUserEmail.setId(sysUser.getId());
        subUserEmail.setSid(adwebSite.getId());
        subUserEmail.setUsername(sysUser.getUsername());
        subUserEmail.setRealname(sysUser.getRealname());
        subUserEmailService.save(subUserEmail);
        // 子账号邮件信息添加
        enquiryEmailMessageList.add(newEmail(adwebEnquiry, sysUser.getEmail()));

        // 修改询盘信息的跟进人信息
        adwebEnquiry.setPrincipalUid(sysUser.getId());
        adwebEnquiryMapper.updateById(adwebEnquiry);

        // 获取主账号
        SysUser mainUser = sysUserService.getById(adwebSite.getUid());
        // 主账号邮件信息添加
        if (mainUser != null && StringUtils.isNotBlank(mainUser.getEmail())) {
          enquiryEmailMessageList.add(newEmail(adwebEnquiry, mainUser.getEmail()));
        }
      }
      log.info("=======发送询盘邮件======");

      enquiryEmailMessageList.forEach(e -> System.out.println(e.getEmail()));
      log.info("需要添加的询盘邮件：{}", FastJsonUtil.toJSONString(enquiryEmailMessageList));
      if (CollectionUtils.isNotEmpty(enquiryEmailMessageList)) {
        LambdaQueryWrapper<EnquiryEmailMessage> enquiryEmailMessageQueryWrapper =
            new LambdaQueryWrapper<>();
        enquiryEmailMessageQueryWrapper.eq(EnquiryEmailMessage::getSiteCode, adwebSite.getCode());
        List<EnquiryEmailMessage> enquiryEmailMessages =
            enquiryEmailMessageService.list(enquiryEmailMessageQueryWrapper);
        List<EnquiryEmailMessage> sameEnquiryEmailMessageList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(enquiryEmailMessages)) {
          for (EnquiryEmailMessage enquiryEmailMessage : enquiryEmailMessages) {
            for (EnquiryEmailMessage enquiryEmailMessage1 : enquiryEmailMessageList) {
              if (enquiryEmailMessage.getEnquiryId().equals(enquiryEmailMessage1.getEnquiryId())
                  && enquiryEmailMessage.getEmail().equals(enquiryEmailMessage1.getEmail())) {
                sameEnquiryEmailMessageList.add(enquiryEmailMessage1);
              }
            }
          }
          enquiryEmailMessageList.removeAll(sameEnquiryEmailMessageList);
        }
        log.info("最终添加的询盘邮件：{}", FastJsonUtil.toJSONString(enquiryEmailMessageList));
        enquiryEmailMessageService.saveBatch(enquiryEmailMessageList);
        try {
          for (EnquiryEmailMessage message : enquiryEmailMessageList) {
            JedisUtil.lpush(redisKey, ObjectUtil.object2Bytes(message.getEnquiryId()));
            JedisUtil.publishMsg(
                EnquiryConstants.ENQUIRY_EMAIL_CHANNEL, EnquiryConstants.ENQUIRY_EMAIL);
          }
          SysUser user = sysUserService.getById(adwebEnquiry.getPrincipalUid());
          if (StringUtils.isNotBlank(adwebEnquiry.getRecordId())) {
            returnSalesperson(adwebSite.getCode(), adwebEnquiry.getRecordId(), user.getUsername());
          }
        } catch (Exception e) {
          log.error("推送发送询盘邮件的消息失败", e);
        }
      }

    } else {
      log.info("=======该垃圾询盘在特殊站点中不发送询盘邮件======{}", adwebEnquiry.getId());
    }
  }

  /**
   * 根据询盘信息创建邮件发送记录
   *
   * @param adwebEnquiry 询盘信息
   * @param email 接收邮箱
   * @return
   */
  private static EnquiryEmailMessage newEmail(AdwebEnquiry adwebEnquiry, String email) {
    EnquiryEmailMessage enquiryEmailMessage = new EnquiryEmailMessage();
    enquiryEmailMessage.setEnquiryId(adwebEnquiry.getId());
    enquiryEmailMessage.setEmail(email);
    enquiryEmailMessage.setFromEmail(adwebEnquiry.getFromEmail());
    enquiryEmailMessage.setCreateTime(new Date());
    enquiryEmailMessage.setSiteCode(adwebEnquiry.getSiteCode());
    enquiryEmailMessage.setSendStatus(EnquirySendStatus.WAIT_SEND);
    enquiryEmailMessage.setSendErrorNum(0);

    return enquiryEmailMessage;
  }

  /**
   * 通过询盘中的ip字段，获取国家信息并放在询盘对象种
   *
   * @param enquiry 询盘对象
   */
  private void setCountryByIp(AdwebEnquiry enquiry) {
    CountryAreaApiDto countryAreaApiDto = geoIpUtil.getCountryAndAreaByIp(enquiry.getFromIp());
    if (countryAreaApiDto != null) {
      String countryCode = countryAreaApiDto.getCountryIsoCode();
      if ("TW".equals(countryCode)) {
        countryCode = "CN";
      }
      String countryName = countryAreaApiDto.getCountryZhCN();
      if ("中华民国".equals(countryName)) {
        countryName = "中国";
      }
      enquiry.setCountryCode(countryCode);
      enquiry.setCountryName(countryName);
      if (StringUtils.isBlank(enquiry.getCountry())) {
        enquiry.setCountry(countryName);
      }
    } else {
      enquiry.setCountryCode("0000");
      enquiry.setCountryName("其它");
    }
  }

  /**
   * 根据询盘对象和站点，构建一个本系统中的询盘对象
   *
   * @param enquiryDto 询盘对象
   * @param adwebSite 询盘所属站点
   * @return 本系统中的询盘队形
   */
  private AdwebEnquiry createEnquiry(EnquiryDTO enquiryDto, AdwebSite adwebSite) {
    AdwebEnquiry adwebEnquiry = new AdwebEnquiry();
    adwebEnquiry.setId(snowflakeIdUtil.nextId());
    adwebEnquiry.setCtime(new Date());
    adwebEnquiry.setStatus(1);
    adwebEnquiry.setVisitId(enquiryDto.getVisitId());
    adwebEnquiry.setSiteId(adwebSite.getId());
    adwebEnquiry.setUid(adwebSite.getUid());
    adwebEnquiry.setFromIp(enquiryDto.getFromIp());
    adwebEnquiry.setFromEmail(enquiryDto.getFromEmail());
    adwebEnquiry.setPluginName(enquiryDto.getPluginName());
    adwebEnquiry.setRecordId(enquiryDto.getRecordId());
    Date modifyRecordCtime =
        StringUtils.isNotBlank(enquiryDto.getFromIp())
            ? geoIpUtil.getLocalhostTime(enquiryDto.getFromIp(), enquiryDto.getRecordCtime())
            : new Date();
    adwebEnquiry.setModifyRecordCtime(modifyRecordCtime);
    adwebEnquiry.setRecordCtime(enquiryDto.getRecordCtime());
    adwebEnquiry.setSiteCode(adwebSite.getCode());
    adwebEnquiry.setModular(enquiryDto.getPluginName());
    adwebEnquiry.setSysEffective(NumConstant.ONE);
    adwebEnquiry.setUserEffective(NumConstant.TWO);
    adwebEnquiry.setSiteHost(enquiryDto.getSiteHost());
    adwebEnquiry.setNo(
        "xp"
            + DateUtil.formatDate(new Date(), DateUtil.DATE_FORMAT)
            + adwebOpenApiService.loadNo(adwebSite.getCode()));

    // 外部编号暂无作用，且多查询一次数据库，暂时注释
    //        adwebEnquiry.setNoOut(DateUtil.dateToString(new Date(),
    // DateUtil.DATE_FORMAT_THREE) + adwebOpenApiService.loadOutNoByUser(adwebSite.getUid()));
    adwebEnquiry.setWasteEnquiry(0);
    adwebEnquiry.setRequestTime(new Date());
    adwebEnquiry.setAcquireMessageTime(adwebSite.getEnquiryMessageTime());
    adwebEnquiry.setIsEnquirySync("enquirySync");

    adwebEnquiry.setFromSource(enquiryDto.getFromSource());
    adwebEnquiry.setEnquiryFile(enquiryDto.getEnquiryFile());
    adwebEnquiry.setCountryCode(enquiryDto.getCountryCode());

    QueryWrapper<AdwebCountry> queryWrapper = new QueryWrapper<>();
    queryWrapper.eq("country_iso_code", enquiryDto.getCountryCode());

    AdwebCountry adwebCountry = adwebCountryService.getOne(queryWrapper);

    if (adwebCountry != null) {
      adwebEnquiry.setCountry(adwebCountry.getCountryName());
      adwebEnquiry.setCountryName(adwebCountry.getCountryName());
    }

    ObjectMapper objectMapper = new ObjectMapper();
    try {
      String jsonString = objectMapper.writeValueAsString(enquiryDto.getCartItems());
      adwebEnquiry.setCartItems(jsonString);
    } catch (JsonProcessingException e) {
      // 可以选择记录更详细的堆栈跟踪信息
      log.error("Error serializing cart items to JSON: {}", e.getMessage(), e);
    }
    return adwebEnquiry;
  }

  /**
   * 解析form，将form中的字段加入到询盘对象种
   *
   * @param enquiry 询盘对象
   * @param form 询盘表单信息
   */
  private void parseFormAndSetFields(AdwebEnquiry enquiry, String form, AdwebSite adwebSite) {
    long startTime = System.currentTimeMillis();

    List<String> nameKeys = this.getFormKeys("wp-name");
    List<String> emailKeys = this.getFormKeys("wp-email");
    List<String> messageKeys = this.getFormKeys("wp-content");
    List<String> companyKeys = this.getFormKeys("wp-company");
    List<String> fromPageKeys = this.getFormKeys("wp-page");
    List<String> phoneKeys = this.getFormKeys("wp-phone");
    List<String> whatsAppKeys = this.getFormKeys("wp-whatsapp");
    List<String> countryKeys = this.getFormKeys("wp-country");
    List<String> customerIpKeys = this.getFormKeys("wp-customerip");
    List<String> addressKeys = this.getFormKeys("wp-address");

    List<String> errorObjNames = new ArrayList<>();
    Collection<JSONObject> formItems = null;
    Map<String, JSONObject> formObj = FastJsonUtil.parseObject(form, Map.class);
    if (formObj == null || formObj.size() == 0) {
      List<JSONObject> formList = FastJsonUtil.parseObject(form, List.class);
      if (CollectionUtils.isNotEmpty(formList)) {
        formItems = formList;
      }
    } else {
      formItems = formObj.values();
    }

    List<EnquirySpecialFields> specialFieldsList = new ArrayList<>();
    for (JSONObject formItem : formItems) {
      String formItemName = formItem.getString("name");
      String formItemValue = formItem.getString("value");

      if (contains(nameKeys, formItemName)) {
        enquiry.setName(formItemValue);
      } else if (contains(emailKeys, formItemName)) {
        enquiry.setEmail(formItemValue);
      } else if (contains(messageKeys, formItemName)) {
        enquiry.setMessage(formItemValue);
      } else if (contains(companyKeys, formItemName)) {
        enquiry.setCompany(formItemValue);
      } else if (contains(fromPageKeys, formItemName)) {
        enquiry.setFromPage(formItemValue);
      } else if (contains(phoneKeys, formItemName)) {
        enquiry.setPhone(formItemValue);
      } else if (contains(whatsAppKeys, formItemName)) {
        enquiry.setWhatsApp(formItemValue);
      } else if (contains(countryKeys, formItemName)) {
        enquiry.setCountry(formItemValue);
      } else if (contains(customerIpKeys, formItemName)) {
        enquiry.setCustomerIp(formItemValue);
      } else if (contains(addressKeys, formItemName)) {
        enquiry.setAddress(formItemValue);
      } else {
        // 异常字段
        errorObjNames.add(formItemName);
        EnquirySpecialFields enquirySpecialFields = new EnquirySpecialFields();
        enquirySpecialFields.setField(formItemName);
        enquirySpecialFields.setValue(formItemValue);
        specialFieldsList.add(enquirySpecialFields);
      }
    }

    String specialField = FastJsonUtil.toJSONString(specialFieldsList);
    enquiry.setSpecialField(specialField);

    if (!errorObjNames.isEmpty()
        || enquiry.getRecordCtime() == null
        || StringUtil.isEmpty(enquiry.getMessage())
        || StringUtil.isEmpty(enquiry.getEmail())
        || StringUtil.isEmpty(enquiry.getFromPage())) {
      // 发送飞书告警
      StringBuilder msg =
          new StringBuilder(
              "询盘异常,\n询盘id："
                  + enquiry.getId()
                  + ",\n站点名称："
                  + adwebSite.getName()
                  + ",\n站点域名："
                  + enquiry.getSiteHost()
                  + ",\n来源ip："
                  + enquiry.getFromIp());
      if (!errorObjNames.isEmpty()) {
        msg.append(",\n字段异常,存在异常字段：");
        for (String errorObjName : errorObjNames) {
          msg.append(errorObjName).append("、");
        }
        msg.deleteCharAt(msg.length() - 1);
      }
      if (enquiry.getRecordCtime() == null) {
        msg.append(",\n询盘发送时间为空");
      }
      if (StringUtil.isEmpty(enquiry.getMessage())) {
        msg.append(",\n询盘内容为空");
      }
      if (StringUtil.isEmpty(enquiry.getEmail())) {
        msg.append(",\n邮箱内容为空");
      }
      if (StringUtil.isEmpty(enquiry.getFromPage())) {
        msg.append(",\n来源页面为空");
      }
      // 同一个站点，一天最多报错一次
      String flag =
          adwebRedisUtil.getString(
              "forbidden_send_error_msg_by_enquiry_field_" + adwebSite.getDomain());
      if (StringUtils.isNotBlank(flag)) {
        log.info("同一个站点，一天最多报错一次，站点域名：" + adwebSite.getDomain());
        return;
      }
      adwebRedisUtil.set(
          "forbidden_send_error_msg_by_enquiry_field_" + adwebSite.getDomain(),
          "forbidden",
          60 * 60 * 24);

      // 异常入库
      SysException exception = new SysException();
      exception.setType(0);
      exception.setFunctionModule("询盘字段配置异常");
      exception.setExceptionDetail(msg.toString());
      exception.setCreateTime(new Date());
      exception.setStatus(1);
      exception.setHandle(0);
      exception.setPriority(1);
      sysExceptionService.save(exception);

      long diffTime = System.currentTimeMillis() - startTime;
      String className = this.getClass().getName();
      String methodName = Thread.currentThread().getStackTrace()[1].getMethodName();
      feishuService.sendEnquiryRobot(
          (float) diffTime / 1000, "wpforms-询盘警告：" + 0, className, methodName, String.valueOf(msg));
    }
  }

  private boolean contains(List<String> keys, String key) {
    if (CollectionUtils.isEmpty(keys) || StringUtil.isEmpty(key)) {
      return false;
    }
    key = key.replace(":", "").replace("：", "").replaceAll(" {2,}", " ").toLowerCase().trim();
    return keys.contains(key);
  }

  /**
   * 根据字典code，获取相应数据在json中所有可能的key，用来解析json
   *
   * @param dictCode 字典code
   * @return key的集合
   */
  private List<String> getFormKeys(String dictCode) {
    String formKey = CacheConstant.SYS_DICT_CACHE + "::" + dictCode;
    Object obj = adwebRedisUtil.get(formKey);
    if (obj != null) {
      List<String> keys = (List<String>) obj;
      return keys;
    }
    List<SysDictItem> dictItems = sysAdwebApiImpl.selectItemsByDictCode(dictCode);
    List<String> keys = new ArrayList<>();
    if (CollectionUtils.isNotEmpty(dictItems)) {
      for (SysDictItem item : dictItems) {
        String value = item.getItemValue();
        if (StringUtil.isEmpty(value)) {
          continue;
        }
        value = value.replaceAll(" {2,}", " ").toLowerCase().trim();
        if (!keys.contains(value)) {
          keys.add(value);
        }
      }
    }
    adwebRedisUtil.set(formKey, keys, 60 * 60);
    return keys;
  }

  /**
   * 判断邮件是否是垃圾询盘 =》 基于站点黑名单 1、邮件过滤 2、根据邮件是否包含外部url过滤 3、根据关键词过滤
   *
   * @param adwebEnquiryList 询盘列表
   */
  private void judgeWasteEnquiryBySiteId(
      List<AdwebEnquiry> adwebEnquiryList, AdwebSite adwebSite, List<String> blackWordList) {

    // 站点邮箱黑名单
    List<String> siteBlackEmailList = getSiteBlackEmailList(adwebSite);
    // 公共邮箱黑名单
    List<String> publicBlackEmailList = getPublicBlackEmailList();

    // 处理可能的 null 返回值
    siteBlackEmailList = siteBlackEmailList == null ? Collections.emptyList() : siteBlackEmailList;
    publicBlackEmailList =
        publicBlackEmailList == null ? Collections.emptyList() : publicBlackEmailList;

    // 合并两个列表并去重
    List<String> blackEmailList =
        Stream.concat(siteBlackEmailList.stream(), publicBlackEmailList.stream())
            .distinct()
            .collect(Collectors.toList());

    log.info("公共邮箱黑名单长度：{}", publicBlackEmailList.size());
    log.info("站点邮箱黑名单长度：{}", siteBlackEmailList.size());
    log.info("全部邮箱黑名单长度：{}", blackEmailList.size());

    // 站点邮箱白名单
    List<String> siteWhiteEmailList = getSiteWhiteEmailList(adwebSite);
    // 公共邮箱白名单
    List<String> publicWhiteEmailList = getPublicWhiteEmailList();

    // 处理可能的 null 返回值
    siteWhiteEmailList = siteWhiteEmailList == null ? Collections.emptyList() : siteWhiteEmailList;
    publicWhiteEmailList =
        publicWhiteEmailList == null ? Collections.emptyList() : publicWhiteEmailList;

    // 合并两个列表并去重
    List<String> whiteEmailList =
        Stream.concat(siteWhiteEmailList.stream(), publicWhiteEmailList.stream())
            .distinct()
            .collect(Collectors.toList());

    log.info("公共邮箱白名单长度：{}", siteWhiteEmailList.size());
    log.info("站点邮箱白名单长度：{}", publicWhiteEmailList.size());
    log.info("全部邮箱白名单长度：{}", whiteEmailList.size());

    // 公共ip黑名单
    List<String> publicBlackIpList = getPublicBlackIpList();
    // 站点ip黑名单
    List<String> siteBlackIpList = getSiteBlackIpList(adwebSite);

    // 处理可能的 null 返回值
    siteBlackIpList = siteBlackIpList == null ? Collections.emptyList() : siteBlackIpList;
    publicBlackIpList = publicBlackIpList == null ? Collections.emptyList() : publicBlackIpList;
    List<String> blackIpList =
        Stream.concat(siteBlackIpList.stream(), publicBlackIpList.stream())
            .distinct()
            .collect(Collectors.toList());

    log.info("公共ip黑名单长度：{} 个", publicBlackIpList.size());
    log.info("站点ip黑名单长度：{} 个", siteBlackIpList.size());
    log.info("全部ip黑名单长度：{}", blackIpList.size());

    // 公共Ip白名单
    List<String> publicWhiteIpList = getPublicWhiteIpList();
    // 站点Ip白名单
    List<String> siteWhiteIpList = getSiteWhiteIpList(adwebSite);

    // 处理可能的 null 返回值
    siteWhiteIpList = siteWhiteIpList == null ? Collections.emptyList() : siteWhiteIpList;
    publicWhiteIpList = publicWhiteIpList == null ? Collections.emptyList() : publicWhiteIpList;

    List<String> whiteIpList =
        Stream.concat(siteWhiteIpList.stream(), publicWhiteIpList.stream())
            .distinct()
            .collect(Collectors.toList());

    log.info("公共ip白名单长度：{}", publicWhiteIpList.size());
    log.info("站点ip白名单长度：{}", siteWhiteIpList.size());
    log.info("全部ip白名单长度：{}", whiteIpList.size());

    // 站点询盘规则黑名单
    List<AdwebSiteEnquiryRule> siteBlackRuleList = geSiteBlackRuleList(adwebSite);
    log.info("站点关键词黑名单长度：{}", siteBlackRuleList.size());

    // 判断上一封询盘是否是垃圾询盘
    AdwebEnquiry isJudgeWasteEnquiry = new AdwebEnquiry();

    // 判断是否垃圾询盘
    outerloop:
    for (AdwebEnquiry adwebEnquiry : adwebEnquiryList) {
      // 没有message的无法判断，认为是普通询盘
      if (StringUtil.isEmpty(adwebEnquiry.getMessage())) {
        markAsWasteEnquiry(adwebEnquiry, EMPTY_MESSAGE_REASON, "");
        logInfo(adwebEnquiry, "通过——询盘内容为空——检测到垃圾询盘");
        continue;
      }

      // 该分支用于判断上一封邮件是否为垃圾询盘
      dealMultipleRepeatInfoEnquiry(
          siteBlackEmailList, siteBlackIpList, isJudgeWasteEnquiry, adwebSite);
      isJudgeWasteEnquiry = adwebEnquiry;

      // 判断询盘邮箱黑名单
      boolean shotTargetEmail =
          dealEmailBlackList(
              adwebEnquiry,
              blackEmailList,
              publicBlackEmailList,
              siteWhiteEmailList,
              siteBlackEmailList);

      // 命中邮箱黑名单后跳出循环
      if (shotTargetEmail) {
        continue outerloop;
      }

      // 判断是否为ip黑名单
      boolean shotTargetIp =
          dealIpBlackList(
              adwebEnquiry,
              blackIpList,
              publicBlackIpList,
              publicWhiteIpList,
              siteWhiteIpList,
              siteBlackIpList);
      // 命中IP黑名单后跳出循环
      if (shotTargetIp) {
        continue;
      }

      // 判断邮箱是否包含外部链接
      boolean isShotTargetLink = dealMessageContentUrl(adwebEnquiry, adwebSite);
      // 如果询盘内容包含外部链接，命中处理后跳出循环
      if (isShotTargetLink) {
        continue;
      }

      // 判断询盘是否包含关键词
      boolean isShotTargetKeyword =
          dealMessageContentKeyWords(adwebEnquiry, blackWordList, siteBlackRuleList);

      // 如果询盘内容包含外关键词黑命中的关键词，命中处理后跳出循环
      if (isShotTargetKeyword) {
        continue outerloop;
      }

      // 分词判断
      if (validateWordLength(adwebEnquiry.getMessage())) {
        String reason = String.format(LONG_WORD_REASON);
        markAsWasteEnquiry(adwebEnquiry, reason, "other");
        logInfo(adwebEnquiry, reason);
        continue;
      }

      boolean isBlackFlag = false;

      // 询盘为正常询盘
      // 判断十分钟和一天内同个站点是否有相同的邮箱
      if (StringUtils.isNotBlank(adwebEnquiry.getEmail())
          && !whiteEmailList.contains(adwebEnquiry.getEmail().toLowerCase())) {
        isBlackFlag =
            dealShortSameEmail(adwebEnquiry, blackEmailList, adwebSite, siteBlackEmailList);
      }

      // 判断十分钟和一天内同个站点是否有相同的Ip
      if (StringUtils.isNotBlank(adwebEnquiry.getFromIp())
          && !whiteIpList.contains(adwebEnquiry.getFromIp())) {
        isBlackFlag = dealShortSameIp(adwebEnquiry, blackIpList, adwebSite, siteBlackIpList);
      }
      if (isBlackFlag) {
        continue;
      }
      // 设置为正常询盘
      adwebEnquiry.setWasteEnquiry(0);
      adwebEnquiry.setUserEffective(2);
      // 此处为正常询盘，清除疑似垃圾询盘可能性
      isJudgeWasteEnquiry = null;
      log.info("询盘id是:{}, 站点code是:{}, 不是垃圾询盘", adwebEnquiry.getId(), adwebEnquiry.getSiteCode());
    }

    // 判断最后一个询盘是否为垃圾询盘
    isPreviousOneWasteEnquiry(blackEmailList, blackIpList, isJudgeWasteEnquiry);

    // 更新redis
    if (CollectionUtils.isNotEmpty(siteBlackEmailList)) {
      adwebRedisUtil.set(SiteBlackEmailKey + "::" + adwebSite.getId(), siteBlackEmailList, 86400);
    }
    if (CollectionUtils.isNotEmpty(siteBlackIpList)) {
      adwebRedisUtil.set(SiteBlackIpKey + "::" + adwebSite.getId(), siteBlackIpList, 86400);
    }
    if (CollectionUtils.isNotEmpty(siteWhiteEmailList)) {
      adwebRedisUtil.set(
          SiteWhiteEmailListKey + "::" + adwebSite.getId(), siteWhiteEmailList, 86400);
    }
    if (CollectionUtils.isNotEmpty(siteWhiteIpList)) {
      adwebRedisUtil.set(SiteWhiteIpListKey + "::" + adwebSite.getId(), siteWhiteIpList, 86400);
    }
    // 公共
    if (CollectionUtils.isNotEmpty(publicBlackEmailList)) {
      adwebRedisUtil.set(BlackEmailKey, publicBlackEmailList, 86400);
    }
    if (CollectionUtils.isNotEmpty(publicBlackIpList)) {
      adwebRedisUtil.set(BlackIpKey, publicBlackIpList, 86400);
    }
    if (CollectionUtils.isNotEmpty(publicWhiteEmailList)) {
      adwebRedisUtil.set(WhiteEmailListKey, publicWhiteEmailList, 86400);
    }
    if (CollectionUtils.isNotEmpty(publicWhiteIpList)) {
      adwebRedisUtil.set(WhiteIpListKey, publicWhiteIpList, 86400);
    }
  }

  /** 询盘处理邮箱黑名单 */
  private boolean dealEmailBlackList(
      AdwebEnquiry adwebEnquiry,
      List<String> blackEmailList,
      List<String> publicBlackEmailList,
      List<String> siteWhiteEmailList,
      List<String> siteBlackEmailList) {
    if (CollectionUtils.isNotEmpty(blackEmailList)
        || StringUtils.isNotBlank(adwebEnquiry.getEmail())) {
      log.info(" --- 开始判断询盘邮箱黑名单 --- 当前询盘Id:{}", adwebEnquiry.getId());
      String email = adwebEnquiry.getEmail().toLowerCase();
      String message =
          adwebEnquiry
              .getMessage()
              .replaceAll("[\\n\\t]", " ")
              .replaceAll(" {2,}", " ")
              .toLowerCase();

      // 公共邮箱黑名单
      String containPublicBlackStr = "";
      boolean isContainPublicBlack = false;
      if (CollectionUtils.isNotEmpty(publicBlackEmailList)) {
        for (String blackEmail : publicBlackEmailList) {
          String allBlackEmail = blackEmail;
          if (allBlackEmail.contains("*")) {
            allBlackEmail = allBlackEmail.substring(1, allBlackEmail.length() - 1);
            if (email.contains(allBlackEmail) || message.contains(allBlackEmail)) {
              containPublicBlackStr = blackEmail;
              isContainPublicBlack = true;
              break;
            }
          } else {
            if (email.equals(allBlackEmail) || message.equals(allBlackEmail)) {
              containPublicBlackStr = blackEmail;
              isContainPublicBlack = true;
              break;
            }
          }
        }
      }
      log.info("是否是公共黑名单：{}", isContainPublicBlack);
      log.info("匹配的公共黑名单：{}", containPublicBlackStr);

      // 站点白名单
      boolean isContainSiteWhite = false;
      boolean isSiteAllContain = true;
      String containSiteWhiteStr = "";
      if (CollectionUtils.isNotEmpty(siteWhiteEmailList)) {
        for (String whiteEmail : siteWhiteEmailList) {
          String compareWhiteEmail = whiteEmail;
          if (compareWhiteEmail.contains("*")) {
            compareWhiteEmail = whiteEmail.substring(1, whiteEmail.length() - 1);
            if (email.contains(compareWhiteEmail) || message.contains(compareWhiteEmail)) {
              containSiteWhiteStr = compareWhiteEmail;
              isContainSiteWhite = true;
              isSiteAllContain = false;
              break;
            }
          } else {
            if (email.equals(compareWhiteEmail) || message.equals(compareWhiteEmail)) {
              containSiteWhiteStr = compareWhiteEmail;
              isContainSiteWhite = true;
              isSiteAllContain = false;
              break;
            }
          }
        }
      }

      log.info("是否是站点白名单：{}", isContainSiteWhite);
      log.info("匹配的站点白名单：{}", containSiteWhiteStr);
      // 站点黑名单
      boolean isContainSiteBlack = false;
      String containSiteBlackStr = "";
      if (CollectionUtils.isNotEmpty(siteBlackEmailList)) {
        for (String blackEmail : siteBlackEmailList) {
          String allBlackEmail = blackEmail;
          if (allBlackEmail.contains("*")) {
            allBlackEmail = allBlackEmail.substring(1, allBlackEmail.length() - 1);
            if (email.contains(allBlackEmail) || message.contains(allBlackEmail)) {
              containSiteBlackStr = allBlackEmail;
              isContainSiteBlack = true;
              isSiteAllContain = false;
              break;
            }
          } else {
            if (email.equals(allBlackEmail) || message.equals(allBlackEmail)) {
              containSiteBlackStr = allBlackEmail;
              isContainSiteBlack = true;
              isSiteAllContain = false;
              break;
            }
          }
        }
      }

      log.info("是否是站点黑名单：{}", isContainSiteBlack);
      log.info("匹配的站点黑名单：{}", containSiteBlackStr);

      // 判断邮箱或者内容即在黑名单里又在白名单里，特殊处理
      if (isContainSiteBlack && isContainSiteWhite) {
        if (containSiteBlackStr.length() <= containSiteWhiteStr.length()) {
          isContainSiteBlack = false;
        }
      }

      log.info("第二次处理是否是站点黑名单：{}", isContainSiteBlack);
      if (isContainPublicBlack || !isSiteAllContain) {

        // 公共邮箱黑名单和站点邮箱黑名单全部命中
        if (isContainPublicBlack && isContainSiteBlack) {
          String reason = String.format(EMAIL_BLACKLIST_REASON, containSiteBlackStr);
          markAsWasteEnquiry(adwebEnquiry, reason, "email");
          logInfo(adwebEnquiry, reason);
          return true;
        }

        // 公共邮箱黑名单命中
        if (isContainPublicBlack && isSiteAllContain) {
          String reason = String.format(EMAIL_PUBLIC_BLACKLIST_REASON, containPublicBlackStr);
          markAsWasteEnquiry(adwebEnquiry, reason, "email");
          logInfo(adwebEnquiry, reason);
          return true;
        }

        // 站点邮箱黑名单命中
        if (!isContainPublicBlack && isContainSiteBlack) {
          String reason = String.format(EMAIL_SITE_BLACKLIST_REASON, containSiteBlackStr);
          markAsWasteEnquiry(adwebEnquiry, reason, "email");
          logInfo(adwebEnquiry, reason);
          return true;
        }
      }
    }

    return false;
  }

  /** 询盘处理IP黑名单 */
  private boolean dealIpBlackList(
      AdwebEnquiry adwebEnquiry,
      List<String> blackIpList,
      List<String> publicBlackIpList,
      List<String> publicWhiteIpList,
      List<String> siteWhiteIpList,
      List<String> siteBlackIpList) {
    if (CollectionUtils.isNotEmpty(blackIpList)
        || StringUtils.isNotBlank(adwebEnquiry.getCustomerIp())) {
      String ip = adwebEnquiry.getCustomerIp();
      // 公共ip黑名单
      boolean isPublicIp = publicBlackIpList.contains(ip) && !publicWhiteIpList.contains(ip);

      // 站点ip黑名单
      boolean isSiteIp = siteBlackIpList.contains(ip) && !siteWhiteIpList.contains(ip);

      // 公共IP黑名单和站点IP黑名单全部命中
      if (isPublicIp && isSiteIp) {
        String reason = String.format(IP_BLACKLIST_REASON, ip);
        markAsWasteEnquiry(adwebEnquiry, reason, "ip");
        logInfo(adwebEnquiry, reason);
        return true;
      }

      // 公共IP黑名单命中
      if (isPublicIp) {
        String reason = String.format(IP_PUBLIC_BLACKLIST_REASON, ip);
        markAsWasteEnquiry(adwebEnquiry, reason, "ip");
        logInfo(adwebEnquiry, reason);
        return true;
      }

      // 站点IP黑名单命中
      if (isSiteIp) {
        String reason = String.format(IP_SITE_BLACKLIST_REASON, ip);
        markAsWasteEnquiry(adwebEnquiry, reason, "ip");
        logInfo(adwebEnquiry, reason);
        return true;
      }
    }

    return false;
  }

  /** 询盘内容处理是否包含外部链接 */
  private boolean dealMessageContentUrl(AdwebEnquiry adwebEnquiry, AdwebSite adwebSite) {
    List<String> urlList = parser(adwebEnquiry.getMessage());
    if (StringUtils.isNotBlank(adwebSite.getDomain())) {
      for (String url : urlList) {

        // 询盘内容包含外部链接命中
        if (!adwebSite.getDomain().contains(url)) {
          String reason = String.format(URL_BLACKLIST_REASON, url);
          markAsWasteEnquiry(adwebEnquiry, reason, "other");
          logInfo(adwebEnquiry, reason);
          return true;
        }
      }
    } else {
      if (CollectionUtils.isNotEmpty(urlList)) {
        String reason = String.format(URL_BLACKLIST_REASON, "空");
        markAsWasteEnquiry(adwebEnquiry, reason, "other");
        logInfo(adwebEnquiry, reason);
        return true;
      }
    }

    return false;
  }

  /** 询盘内容处理是否包含关键词黑名单 */
  private boolean dealMessageContentKeyWords(
      AdwebEnquiry adwebEnquiry,
      List<String> blackWordList,
      List<AdwebSiteEnquiryRule> siteBlackRuleList) {
    String message =
        adwebEnquiry
            .getMessage()
            .replaceAll("[\\n\\t]", " ")
            .replaceAll(" {2,}", " ")
            .toLowerCase();
    if (CollectionUtils.isNotEmpty(blackWordList)
        || CollectionUtils.isNotEmpty(siteBlackRuleList)) {
      // 公共关键词黑名单
      boolean isPublicWord = false;
      String isBlackWord = "";
      for (String blackWord : blackWordList) {
        if (message.contains(blackWord)) {
          isPublicWord = true;
          isBlackWord = blackWord;
        }
      }
      // 站点关键词黑名单
      boolean isSiteWord = false;
      for (AdwebSiteEnquiryRule siteBlackRule : siteBlackRuleList) {
        if (message.contains(siteBlackRule.getWord())) {
          isSiteWord = true;
          isBlackWord = siteBlackRule.getWord();
        }
      }

      // 公共关键词黑名单和站点关键词黑名单全部命中
      if (isPublicWord && isSiteWord) {
        String reason = String.format(KEYWORD_BLACKLIST_REASON, isBlackWord);
        markAsWasteEnquiry(adwebEnquiry, reason, "keyword");
        logInfo(adwebEnquiry, reason);
        return true;
      }

      // 公共关键词黑名单命中
      if (isPublicWord) {
        String reason = String.format(KEYWORD_PUBLIC_BLACKLIST_REASON, isBlackWord);
        markAsWasteEnquiry(adwebEnquiry, reason, "keyword");
        logInfo(adwebEnquiry, reason);
        return true;
      }

      // 站点关键词黑名单命中
      if (isSiteWord) {
        String reason = String.format(KEYWORD_SITE_BLACKLIST_REASON, isBlackWord);
        markAsWasteEnquiry(adwebEnquiry, reason, "keyword");
        logInfo(adwebEnquiry, reason);
        return true;
      }
    }

    return false;
  }

  /** 标记为垃圾询盘 */
  private void markAsWasteEnquiry(AdwebEnquiry adwebEnquiry, String reason, String enquiryType) {
    adwebEnquiry.setWasteEnquiry(1);
    adwebEnquiry.setUserEffective(0);
    adwebEnquiry.setWasteEnquiryType(enquiryType);
    adwebEnquiry.setEffectiveReason(reason);
  }

  /** 询盘记录日志 */
  private void logInfo(AdwebEnquiry adwebEnquiry, String message) {
    log.info("询盘id是:{}, 站点code是:{}, {}", adwebEnquiry.getId(), adwebEnquiry.getSiteCode(), message);
  }

  /** 获取站点关键词黑名单 */
  private List<AdwebSiteEnquiryRule> geSiteBlackRuleList(AdwebSite adwebSite) {
    LambdaQueryWrapper<AdwebSiteEnquiryRule> adwebEnquirySiteRuleQueryWrapper =
        new LambdaQueryWrapper<>();
    adwebEnquirySiteRuleQueryWrapper.eq(AdwebSiteEnquiryRule::getSiteId, adwebSite.getId());
    adwebEnquirySiteRuleQueryWrapper.eq(AdwebSiteEnquiryRule::getStatus, 1);
    adwebEnquirySiteRuleQueryWrapper.eq(AdwebSiteEnquiryRule::getBlackOrWhite, 0);

    return adwebSiteEnquiryRuleService.list(adwebEnquirySiteRuleQueryWrapper);
  }

  /** 站点IP白名单 */
  private List<String> getSiteWhiteIpList(AdwebSite adwebSite) {
    String siteWhiteIpRedisKey = SiteWhiteIpListKey + "::" + adwebSite.getId();

    List<String> siteWhiteIpList = getListFromRedis(siteWhiteIpRedisKey);

    if (CollectionUtils.isEmpty(siteWhiteIpList)) {
      List<AdwebSiteBlackIp> enquirySiteWhiteIpList =
          adwebSiteBlackIpService.list(
              new LambdaQueryWrapper<AdwebSiteBlackIp>()
                  .ne(AdwebSiteBlackIp::getStatus, 0)
                  .eq(AdwebSiteBlackIp::getBlackOrWhite, 1)
                  .eq(AdwebSiteBlackIp::getSiteId, adwebSite.getId())
                  .select(AdwebSiteBlackIp::getIp));
      if (CollectionUtils.isNotEmpty(enquirySiteWhiteIpList)) {
        siteWhiteIpList =
            enquirySiteWhiteIpList.stream()
                .map(AdwebSiteBlackIp::getIp)
                .collect(Collectors.toList());
        adwebRedisUtil.set(siteWhiteIpRedisKey, siteWhiteIpList, 60 * 60 * 24);
      } else {
        siteWhiteIpList = new ArrayList<>();
      }
    }

    return siteWhiteIpList;
  }

  /** 公共IP白名单列表 */
  private List<String> getPublicWhiteIpList() {
    List<String> publicWhiteIpList = getListFromRedis(WhiteIpListKey);
    if (publicWhiteIpList == null) {
      List<AdwebPublicBlackIp> enquiryPublicWhiteIpList =
          adwebPublicBlackIpService.list(
              new LambdaQueryWrapper<AdwebPublicBlackIp>()
                  .ne(AdwebPublicBlackIp::getStatus, 0)
                  .eq(AdwebPublicBlackIp::getBlackOrWhite, 1)
                  .select(AdwebPublicBlackIp::getIp));
      if (CollectionUtils.isNotEmpty(enquiryPublicWhiteIpList)) {
        publicWhiteIpList =
            enquiryPublicWhiteIpList.stream()
                .map(AdwebPublicBlackIp::getIp)
                .collect(Collectors.toList());
        adwebRedisUtil.set(WhiteIpListKey, publicWhiteIpList, 60 * 60 * 24);
      } else {
        publicWhiteIpList = new ArrayList<>();
      }
    }

    return publicWhiteIpList;
  }

  /** 获取公共黑名单IP列表 */
  private List<String> getPublicBlackIpList() {
    List<String> publicBlackIpList = getListFromRedis(BlackIpKey);

    if (CollectionUtils.isEmpty(publicBlackIpList)) {
      List<AdwebPublicBlackIp> enquiryPublicBlackIpList =
          adwebPublicBlackIpService.list(
              new LambdaQueryWrapper<AdwebPublicBlackIp>()
                  .ne(AdwebPublicBlackIp::getStatus, 0)
                  .eq(AdwebPublicBlackIp::getBlackOrWhite, 0)
                  .select(AdwebPublicBlackIp::getIp));
      if (CollectionUtils.isNotEmpty(enquiryPublicBlackIpList)) {
        publicBlackIpList =
            enquiryPublicBlackIpList.stream()
                .map(AdwebPublicBlackIp::getIp)
                .collect(Collectors.toList());
        adwebRedisUtil.set(BlackIpKey, publicBlackIpList, 60 * 60 * 24);
      } else {
        publicBlackIpList = new ArrayList<>();
      }
    }

    return publicBlackIpList;
  }

  /** 获取站点黑名单IP列表 */
  private List<String> getSiteBlackIpList(AdwebSite adwebSite) {
    String siteBlackIpRedisKey = SiteBlackIpKey + "::" + adwebSite.getId();

    List<String> siteBlackIpList = getListFromRedis(siteBlackIpRedisKey);

    if (CollectionUtils.isEmpty(siteBlackIpList)) {
      List<AdwebSiteBlackIp> enquirySiteBlackIpList =
          adwebSiteBlackIpService.list(
              new LambdaQueryWrapper<AdwebSiteBlackIp>()
                  .ne(AdwebSiteBlackIp::getStatus, 0)
                  .eq(AdwebSiteBlackIp::getBlackOrWhite, 0)
                  .eq(AdwebSiteBlackIp::getSiteId, adwebSite.getId())
                  .select(AdwebSiteBlackIp::getIp));
      if (CollectionUtils.isNotEmpty(enquirySiteBlackIpList)) {
        siteBlackIpList =
            enquirySiteBlackIpList.stream()
                .map(AdwebSiteBlackIp::getIp)
                .collect(Collectors.toList());
        adwebRedisUtil.set(siteBlackIpRedisKey, siteBlackIpList, 60 * 60 * 24);
      } else {
        siteBlackIpList = new ArrayList<>();
      }
    }

    return siteBlackIpList;
  }

  /** 获取站点黑名单邮箱列表 */
  private List<String> getSiteBlackEmailList(AdwebSite adwebSite) {
    String siteBlackEmailRedisKey = SiteBlackEmailKey + "::" + adwebSite.getId();
    List<String> siteBlackEmailList = getListFromRedis(siteBlackEmailRedisKey);

    if (CollectionUtils.isEmpty(siteBlackEmailList)) {
      enquirySiteBlackEmailService.saveRedisSiteBlackEmailBySiteId(adwebSite.getId(), 0);
      siteBlackEmailList = getListFromRedis(siteBlackEmailRedisKey);
      if (CollectionUtils.isEmpty(siteBlackEmailList)) {
        siteBlackEmailList = new ArrayList<>();
      }
    }

    return siteBlackEmailList;
  }

  /** 获取站点白名单邮箱列表 */
  private List<String> getSiteWhiteEmailList(AdwebSite adwebSite) {
    String siteWhiteEmailRedisKey = SiteWhiteEmailListKey + "::" + adwebSite.getId();

    List<String> siteWhiteEmailList = getListFromRedis(siteWhiteEmailRedisKey);

    if (CollectionUtils.isEmpty(siteWhiteEmailList)) {
      enquirySiteBlackEmailService.saveRedisSiteBlackEmailBySiteId(adwebSite.getId(), 0);
      siteWhiteEmailList = getListFromRedis(siteWhiteEmailRedisKey);
      if (CollectionUtils.isEmpty(siteWhiteEmailList)) {
        siteWhiteEmailList = new ArrayList<>();
      }
    }

    return siteWhiteEmailList;
  }

  /** 获取公共邮箱黑名单 */
  private List<String> getPublicBlackEmailList() {
    List<String> publicBlackEmailList = getListFromRedis(BlackEmailKey);

    if (CollectionUtils.isEmpty(publicBlackEmailList)) {
      List<AdwebPublicBlackEmail> enquiryPublicBlackEmailList =
          adwebPublicBlackEmailService.list(
              new LambdaQueryWrapper<AdwebPublicBlackEmail>()
                  .ne(AdwebPublicBlackEmail::getStatus, 0)
                  .eq(AdwebPublicBlackEmail::getBlackOrWhite, 0)
                  .isNotNull(AdwebPublicBlackEmail::getEmail)
                  .select(AdwebPublicBlackEmail::getEmail));

      if (CollectionUtils.isNotEmpty(enquiryPublicBlackEmailList)) {
        publicBlackEmailList =
            enquiryPublicBlackEmailList.stream()
                .map(AdwebPublicBlackEmail::getEmail)
                .filter(StringUtil::isNotEmpty)
                .collect(Collectors.toList());
        adwebRedisUtil.set(BlackEmailKey, publicBlackEmailList, 60 * 60 * 24);
      } else {
        publicBlackEmailList = new ArrayList<>();
      }
    } else {
      publicBlackEmailList =
          publicBlackEmailList.stream().filter(s -> !s.isEmpty()).collect(Collectors.toList());
    }

    return publicBlackEmailList;
  }

  /** 获取公共邮箱白名单 */
  private List<String> getPublicWhiteEmailList() {
    List<String> publicWhiteEmailList = getListFromRedis(WhiteEmailListKey);

    if (CollectionUtils.isEmpty(publicWhiteEmailList)) {
      List<AdwebPublicBlackEmail> enquiryPublicWhiteEmailList =
          adwebPublicBlackEmailService.list(
              new LambdaQueryWrapper<AdwebPublicBlackEmail>()
                  .ne(AdwebPublicBlackEmail::getStatus, 0)
                  .eq(AdwebPublicBlackEmail::getBlackOrWhite, 1)
                  .isNotNull(AdwebPublicBlackEmail::getEmail)
                  .select(AdwebPublicBlackEmail::getEmail));

      if (CollectionUtils.isNotEmpty(enquiryPublicWhiteEmailList)) {
        publicWhiteEmailList =
            enquiryPublicWhiteEmailList.stream()
                .map(AdwebPublicBlackEmail::getEmail)
                .filter(StringUtil::isNotEmpty)
                .collect(Collectors.toList());
        adwebRedisUtil.set(WhiteEmailListKey, publicWhiteEmailList, 60 * 60 * 24);
      } else {
        publicWhiteEmailList = new ArrayList<>();
      }
    } else {
      publicWhiteEmailList =
          publicWhiteEmailList.stream().filter(s -> !s.isEmpty()).collect(Collectors.toList());
    }

    return publicWhiteEmailList;
  }

  /** 从redis中获取List<String> */
  private List<String> getListFromRedis(String redisKey) {
    try {
      Object result = adwebRedisUtil.get(redisKey);
      if (result instanceof List<?>) {
        @SuppressWarnings("unchecked")
        List<String> list = (List<String>) result;
        return list;
      } else {
        return new ArrayList<>();
      }
    } catch (Exception e) {
      // Log the exception and return an empty list as a fallback
      log.error("Failed to retrieve data from Redis: {}", e.getMessage());
      return new ArrayList<>();
    }
  }

  /**
   * 验证文本中的最长单词长度是否超过限制 此方法用于确保文本中的任何单词长度不超过系统定义的最大值 它首先检查文本是否为空，然后获取文本中最长单词的长度
   * 接着，它从系统字典中读取最大单词长度的限制，如果未设置，则使用默认值 最后，比较最长单词长度与限制，决定是否返回true
   *
   * @param text 待验证的文本
   * @return 如果文本中的最长单词长度超过限制，则返回true；否则返回false
   */
  private boolean validateWordLength(String text) {
    if (StringUtil.isEmpty(text)) {
      return false;
    }
    int maxWordLength = EnglishAnalyzer.getLongestSize(text);
    int limit;
    SysDictItem dictItem = dictService.getDictItemByCodeAndText("max_word_length", "default");
    if (dictItem == null || StringUtil.isEmpty(dictItem.getItemValue())) {
      limit = 50;
    } else {
      limit = Integer.parseInt(dictItem.getItemValue());
    }
    return (maxWordLength > limit);
  }

  /**
   * 解析给定文本中的URL和邮箱 该方法首先使用正则表达式匹配文本中的URL，包括常见的网络协议方案 然后根据预定义的域名模式或默认模式匹配特定的域名
   * 同时，方法也提取文本中的邮箱地址，并将其拆分为用户名和域名部分 最后，方法确保如果一个邮箱地址被错误地识别为URL，则将其从URL列表中移除
   *
   * @param text 待解析的文本
   * @return 包含所有识别到的URL的列表
   */
  private List<String> parser(String text) {
    Pattern pattern1 = Pattern.compile("(https?|ftp|file)://([-a-zA-Z0-9]+\\.)+[a-zA-Z0-9]+");
    List<String> domainPatterns = getFormKeys("enquiry_domain_pattern");
    String domainPattern = "";
    if (CollectionUtils.isNotEmpty(domainPatterns)) {
      domainPattern = domainPatterns.get(0);
    }
    if (StringUtil.isEmpty(domainPattern)) {
      domainPattern =
          "([-a-zA-Z0-9]+\\.)+(com|cn|net|org|edu|hk|fr|de|uk|eu|tw|ph|my|id|in|ind|co|pk|mx|us"
              + "|ve|ar|eg|online|xyz|me|jp|ly|baz|io|cc|tv|club|xxx|host|ltd|vip|work|fit|pub|love|xin|info|pro|post|coop|int|jobs|cat|travel"
              + "|ru|onion|site|it)+";
    } else {
      domainPattern = "([-a-zA-Z0-9]+\\.)+" + domainPattern;
    }
    Pattern pattern2 = Pattern.compile(domainPattern);

    // 邮箱正则
    Pattern pattern3 =
        Pattern.compile("[a-zA-Z0-9_\\-\\.]+@[a-zA-Z0-9_\\-\\.]+\\.[a-zA-Z0-9_\\-\\.]+");

    // 判断正文中是否存在邮箱
    Matcher matcher = pattern3.matcher(text);
    List<String> emailList = new ArrayList<>();
    while (matcher.find()) {
      String keyWord = matcher.group();
      String lastWord = keyWord.substring(keyWord.length() - 1);
      if (".".equals(lastWord)) {
        keyWord = keyWord.substring(0, keyWord.length() - 1);
      }
      emailList.add(keyWord);
      if (keyWord.lastIndexOf("@") > -1) {
        emailList.add(keyWord.substring(keyWord.lastIndexOf("@") + 1));
        emailList.add(keyWord.substring(0, keyWord.lastIndexOf("@")));
      }
    }

    List<String> urls = new ArrayList<>();

    Matcher matcher1 = pattern1.matcher(text);
    while (matcher1.find()) {
      urls.add(matcher1.group());
    }

    Matcher matcher2 = pattern2.matcher(text);
    while (matcher2.find()) {
      urls.add(matcher2.group());
    }

    // 如果邮箱和 URL 都存在，并且 URL 中不包含 http 和 https，则从 URL 中删除邮箱
    if (CollectionUtils.isNotEmpty(emailList)) {
      Set<String> urlSet = new HashSet<>(urls);
      urlSet.removeAll(
          emailList.stream()
              .filter(
                  email ->
                      urlSet.contains(email) && !email.contains("http") && !email.contains("https"))
              .collect(Collectors.toSet()));
      urls = new ArrayList<>(urlSet);
    }

    urls = urls.stream().distinct().collect(Collectors.toList());
    return urls;
  }

  /** 判断是否时垃圾询盘 */
  private void isPreviousOneWasteEnquiry(
      List<String> blackEmailList, List<String> blackIpList, AdwebEnquiry isJudgeWasteEnquiry) {
    if (isJudgeWasteEnquiry != null && StringUtils.isNotBlank(isJudgeWasteEnquiry.getSiteCode())) {
      log.info("上一封邮件为垃圾询盘,处理中...");
      boolean emailFlag = false;
      String isJudgeWasteEnquiryEmail = isJudgeWasteEnquiry.getEmail().toLowerCase();
      if (StringUtils.isNotBlank(isJudgeWasteEnquiryEmail)
          && CollectionUtils.isNotEmpty(blackEmailList)) {
        for (String blackEmail : blackEmailList) {
          if (isJudgeWasteEnquiry.getEmail().equals(blackEmail)) {
            emailFlag = false;
            break;
          } else if (blackEmail.contains("*")) {
            String generalBlackEmail = blackEmail.substring(1, blackEmail.length() - 1 - 1);
            if (isJudgeWasteEnquiry.getEmail().contains(generalBlackEmail)) {
              emailFlag = false;
              break;
            }
          } else {
            emailFlag = true;
          }
        }
      }

      boolean ipFlag =
          StringUtils.isNotBlank(isJudgeWasteEnquiry.getFromIp())
              && blackIpList != null
              && !blackIpList.contains(isJudgeWasteEnquiry.getFromIp());

      // 该分支用于判断上一封垃圾询盘 不在邮箱黑名单中
      if (emailFlag) {
        Integer notBlackEmailWasteListCount = null;
        if (adwebRedisUtil.hasKey(NotBlackEmailWasteEnquiryKey + isJudgeWasteEnquiry.getEmail())) {
          notBlackEmailWasteListCount =
              (Integer)
                      adwebRedisUtil.get(
                          NotBlackEmailWasteEnquiryKey + isJudgeWasteEnquiry.getEmail())
                  + 1;
        } else {
          notBlackEmailWasteListCount = 1;
        }
        adwebRedisUtil.set(
            NotBlackEmailWasteEnquiryKey + isJudgeWasteEnquiry.getEmail(),
            notBlackEmailWasteListCount,
            60L * 60 * 24 * emailNotBlackListDate);

        // 该分支用于判断是否超过
        if (notBlackEmailWasteListCount > emailNotBlackListNum) {
          AdwebPublicBlackEmail blackEmail =
              new AdwebPublicBlackEmail(
                  isJudgeWasteEnquiry.getEmail(), NumConstant.ONE, NumConstant.ZERO);
          if (adwebPublicBlackEmailService.count(
                  new LambdaQueryWrapper<AdwebPublicBlackEmail>()
                      .eq(AdwebPublicBlackEmail::getEmail, isJudgeWasteEnquiry.getEmail())
                      .ne(AdwebPublicBlackEmail::getStatus, 0))
              == 0) {
            adwebPublicBlackEmailService.save(blackEmail);
            blackEmailList.add(isJudgeWasteEnquiry.getEmail());
            log.info(
                "询盘id是:{}, 站点code是:{}, 检测重复，已标记邮箱, 邮箱是:{}",
                isJudgeWasteEnquiry.getId(),
                isJudgeWasteEnquiry.getSiteCode(),
                isJudgeWasteEnquiry.getEmail());
          }
          adwebRedisUtil.del(NotBlackEmailWasteEnquiryKey + isJudgeWasteEnquiry.getEmail());
        }
      }
      if (ipFlag) {
        Integer notBlackIpWasteListCount = null;
        if (adwebRedisUtil.hasKey(NotBlackIpWasteEnquiryKey + isJudgeWasteEnquiry.getFromIp())) {
          notBlackIpWasteListCount =
              (Integer)
                      adwebRedisUtil.get(
                          NotBlackIpWasteEnquiryKey + isJudgeWasteEnquiry.getFromIp())
                  + 1;
        } else {
          notBlackIpWasteListCount = 1;
        }
        adwebRedisUtil.set(
            NotBlackIpWasteEnquiryKey + isJudgeWasteEnquiry.getFromIp(),
            notBlackIpWasteListCount,
            60L * 60 * 24 * ipNotBlackListDate);

        // 该分支用于判断是否出现超过
        if (notBlackIpWasteListCount > ipNotBlackListNum) {
          AdwebPublicBlackIp blackIp =
              new AdwebPublicBlackIp(
                  isJudgeWasteEnquiry.getFromIp(), NumConstant.ONE, NumConstant.ZERO);
          if (adwebPublicBlackIpService.count(
                  new LambdaQueryWrapper<AdwebPublicBlackIp>()
                      .eq(AdwebPublicBlackIp::getIp, isJudgeWasteEnquiry.getFromIp())
                      .ne(AdwebPublicBlackIp::getStatus, 0))
              == 0) {
            adwebPublicBlackIpService.save(blackIp);
            blackIpList.add(isJudgeWasteEnquiry.getFromIp());
            log.info(
                "询盘id是:{}, 站点code是:{}, 检测重复，已标记ip, ip是:{}",
                isJudgeWasteEnquiry.getId(),
                isJudgeWasteEnquiry.getSiteCode(),
                isJudgeWasteEnquiry.getFromIp());
          }
          adwebRedisUtil.del(NotBlackIpWasteEnquiryKey + isJudgeWasteEnquiry.getFromIp());
        }
      }
    }
  }

  /** 当同步询盘拉取时，处理同个邮箱或者同个ip的询盘达到一定数量后置为黑名单 如果是公共黑名单 */
  private void dealMultipleRepeatInfoEnquiry(
      List<String> siteBlackEmailList,
      List<String> siteBlackIpList,
      AdwebEnquiry isJudgeWasteEnquiry,
      AdwebSite adwebSite) {
    if (isJudgeWasteEnquiry != null && StringUtils.isNotBlank(isJudgeWasteEnquiry.getSiteCode())) {
      log.info("上一封邮件为垃圾询盘,处理中...");
      boolean emailFlag = false;
      String isJudgeWasteEnquiryEmail = isJudgeWasteEnquiry.getEmail().toLowerCase();
      if (StringUtils.isNotBlank(isJudgeWasteEnquiryEmail)
          && CollectionUtils.isNotEmpty(siteBlackEmailList)) {
        for (String blackEmail : siteBlackEmailList) {
          if (isJudgeWasteEnquiry.getEmail().equals(blackEmail)) {
            emailFlag = false;
            break;
          } else if (blackEmail.contains("*")) {
            String generalBlackEmail = blackEmail.substring(1, blackEmail.length() - 1 - 1);
            if (isJudgeWasteEnquiry.getEmail().contains(generalBlackEmail)) {
              emailFlag = false;
              break;
            }
          } else {
            emailFlag = true;
          }
        }
      }

      boolean ipFlag =
          StringUtils.isNotBlank(isJudgeWasteEnquiry.getFromIp())
              && siteBlackIpList != null
              && !siteBlackIpList.contains(isJudgeWasteEnquiry.getFromIp());

      // 该分支用于判断上一封垃圾询盘 不在邮箱黑名单中
      if (emailFlag) {
        Integer notBlackEmailWasteListCount = null;
        if (adwebRedisUtil.hasKey(
            NotBlackEmailWasteSiteEnquiryKey
                + adwebSite.getId()
                + "::"
                + isJudgeWasteEnquiry.getEmail())) {
          notBlackEmailWasteListCount =
              (Integer)
                      adwebRedisUtil.get(
                          NotBlackEmailWasteSiteEnquiryKey
                              + adwebSite.getId()
                              + "::"
                              + isJudgeWasteEnquiry.getEmail())
                  + 1;
        } else {
          notBlackEmailWasteListCount = 1;
        }
        adwebRedisUtil.set(
            NotBlackEmailWasteSiteEnquiryKey
                + adwebSite.getId()
                + "::"
                + isJudgeWasteEnquiry.getEmail(),
            notBlackEmailWasteListCount,
            60L * 60 * 24 * emailNotBlackListDate);

        // 该分支用于判断是否超过
        if (notBlackEmailWasteListCount > emailNotBlackListNum) {
          if (enquirySiteBlackEmailService.count(
                  new LambdaQueryWrapper<AdwebSiteBlackEmail>()
                      .eq(AdwebSiteBlackEmail::getEmail, isJudgeWasteEnquiry.getEmail())
                      .ne(AdwebSiteBlackEmail::getStatus, 0)
                      .eq(AdwebSiteBlackEmail::getSiteId, adwebSite.getId())
                      .eq(AdwebSiteBlackEmail::getBlackOrWhite, 0))
              == 0) {
            siteBlackEmailList.add(isJudgeWasteEnquiry.getEmail());
            log.info(
                "询盘id是:{}, 站点code是:{}, 检测重复，已标记邮箱, 邮箱是:{}",
                isJudgeWasteEnquiry.getId(),
                isJudgeWasteEnquiry.getSiteCode(),
                isJudgeWasteEnquiry.getEmail());
          }
          EffectiveEnquiryParamDto effectiveEnquiryParamDto = new EffectiveEnquiryParamDto();
          effectiveEnquiryParamDto.setSiteId(adwebSite.getId());
          effectiveEnquiryParamDto.setSiteCode(adwebSite.getCode());
          effectiveEnquiryParamDto.setWasteEnquirySeason(isJudgeWasteEnquiry.getEmail());
          enquirySiteBlackEmailService.addBlackEmailByContent(effectiveEnquiryParamDto);
          adwebRedisUtil.del(
              NotBlackEmailWasteSiteEnquiryKey
                  + adwebSite.getId()
                  + "::"
                  + isJudgeWasteEnquiry.getEmail());
        }
      }
      if (ipFlag) {
        Integer notBlackIpWasteListCount = null;
        if (adwebRedisUtil.hasKey(
            NotSiteBlackIpWasteEnquiryKey
                + adwebSite.getId()
                + "::"
                + isJudgeWasteEnquiry.getFromIp())) {
          notBlackIpWasteListCount =
              (Integer)
                      adwebRedisUtil.get(
                          NotSiteBlackIpWasteEnquiryKey
                              + adwebSite.getId()
                              + "::"
                              + isJudgeWasteEnquiry.getFromIp())
                  + 1;
        } else {
          notBlackIpWasteListCount = 1;
        }
        adwebRedisUtil.set(
            NotSiteBlackIpWasteEnquiryKey
                + adwebSite.getId()
                + "::"
                + isJudgeWasteEnquiry.getFromIp(),
            notBlackIpWasteListCount,
            60L * 60 * 24 * ipNotBlackListDate);

        // 该分支用于判断是否出现超过
        if (notBlackIpWasteListCount > ipNotBlackListNum) {
          if (adwebSiteBlackIpService.count(
                  new LambdaQueryWrapper<AdwebSiteBlackIp>()
                      .eq(AdwebSiteBlackIp::getIp, isJudgeWasteEnquiry.getFromIp())
                      .ne(AdwebSiteBlackIp::getStatus, 0)
                      .eq(AdwebSiteBlackIp::getSiteId, adwebSite.getId())
                      .eq(AdwebSiteBlackIp::getBlackOrWhite, 0))
              == 0) {
            siteBlackIpList.add(isJudgeWasteEnquiry.getFromIp());
            log.info(
                "询盘id是:{}, 站点code是:{}, 检测重复，已标记ip, ip是:{}",
                isJudgeWasteEnquiry.getId(),
                isJudgeWasteEnquiry.getSiteCode(),
                isJudgeWasteEnquiry.getFromIp());
          }
          EffectiveEnquiryParamDto effectiveEnquiryParamDto = new EffectiveEnquiryParamDto();
          effectiveEnquiryParamDto.setWasteEnquirySeason(isJudgeWasteEnquiry.getFromIp());
          effectiveEnquiryParamDto.setSiteId(adwebSite.getId());
          effectiveEnquiryParamDto.setSiteCode(adwebSite.getCode());
          adwebSiteBlackIpService.addBlackIpByContent(effectiveEnquiryParamDto);
          adwebRedisUtil.del(
              NotSiteBlackIpWasteEnquiryKey
                  + adwebSite.getId()
                  + "::"
                  + isJudgeWasteEnquiry.getFromIp());
        }
      }
    }
  }

  /** 当同步询盘拉取时，处理10分钟或一天同个邮箱的询盘达到一定数量后置为黑名单 */
  private boolean dealShortSameEmail(
      AdwebEnquiry adwebEnquiry,
      List<String> blackEmailList,
      AdwebSite adwebSite,
      List<String> siteBlackEmailList) {
    Integer tenMin = null;
    String TenMinKey = SiteEmailTenMinKey + adwebSite.getId() + "::" + adwebEnquiry.getEmail();
    String oneDayKey = SiteEmailOneDayKey + adwebSite.getId() + "::" + adwebEnquiry.getEmail();
    if (adwebRedisUtil.hasKey(TenMinKey)) {
      tenMin = (Integer) adwebRedisUtil.get(TenMinKey) + 1;
    } else {
      tenMin = 1;
    }
    adwebRedisUtil.set(TenMinKey, tenMin, 10 * 60);

    if (tenMin > emailTenMinNum) {
      String reason = String.format("系统操作-通过十分钟内重复邮箱检测到垃圾询盘-" + adwebEnquiry.getEmail());
      markAsWasteEnquiry(adwebEnquiry, reason, "email");
      logInfo(adwebEnquiry, reason);
      if (enquirySiteBlackEmailService.count(
              new LambdaQueryWrapper<AdwebSiteBlackEmail>()
                  .eq(AdwebSiteBlackEmail::getEmail, adwebEnquiry.getEmail())
                  .ne(AdwebSiteBlackEmail::getStatus, 0)
                  .eq(AdwebSiteBlackEmail::getSiteId, adwebSite.getId())
                  .eq(AdwebSiteBlackEmail::getBlackOrWhite, 0))
          == 0) {
        siteBlackEmailList.add(adwebEnquiry.getEmail());
        blackEmailList.add(adwebEnquiry.getEmail());

        logInfo(adwebEnquiry, "检测十分钟内重复，已标记邮箱, 邮箱是:" + adwebEnquiry.getEmail());
      }
      // 存储到数据库中
      EffectiveEnquiryParamDto effectiveEnquiryParamDto = new EffectiveEnquiryParamDto();
      effectiveEnquiryParamDto.setSiteId(adwebSite.getId());
      effectiveEnquiryParamDto.setSiteCode(adwebSite.getCode());
      effectiveEnquiryParamDto.setWasteEnquirySeason(adwebEnquiry.getEmail());
      enquirySiteBlackEmailService.addBlackEmailByContent(effectiveEnquiryParamDto);
      // 清除map中的数据
      adwebRedisUtil.del(TenMinKey);
      adwebRedisUtil.del(oneDayKey);
      return true;
    }

    // 如果10分钟内发送了则1天用判断了
    Integer oneDay = null;
    if (adwebRedisUtil.hasKey(oneDayKey)) {
      oneDay = (Integer) adwebRedisUtil.get(oneDayKey) + 1;
    } else {
      oneDay = 1;
    }
    adwebRedisUtil.set(oneDayKey, oneDay, 24 * 60 * 60);

    if (oneDay > emailOneDayNum) {
      String reason = String.format("系统操作-通过一天内重复邮箱检测到垃圾询盘-" + adwebEnquiry.getEmail());
      markAsWasteEnquiry(adwebEnquiry, reason, "email");
      logInfo(adwebEnquiry, reason);

      log.info(
          "'--------dealShortSameEmail -----'===>adwebEnquiry.getEmail():{}",
          adwebEnquiry.getEmail());
      log.info("'--------dealShortSameEmail -----'===>adwebSite.getId():{}", adwebSite.getId());

      if (enquirySiteBlackEmailService.count(
              new LambdaQueryWrapper<AdwebSiteBlackEmail>()
                  .eq(AdwebSiteBlackEmail::getEmail, adwebEnquiry.getEmail())
                  .ne(AdwebSiteBlackEmail::getStatus, 0)
                  .eq(AdwebSiteBlackEmail::getSiteId, adwebSite.getId())
                  .eq(AdwebSiteBlackEmail::getBlackOrWhite, 0))
          == 0) {
        siteBlackEmailList.add(adwebEnquiry.getEmail());
        blackEmailList.add(adwebEnquiry.getEmail());

        logInfo(adwebEnquiry, "检测一天内内重复，已标记邮箱, 邮箱是:" + adwebEnquiry.getEmail());
      }
      // 存储到数据库中
      EffectiveEnquiryParamDto effectiveEnquiryParamDto = new EffectiveEnquiryParamDto();
      effectiveEnquiryParamDto.setSiteId(adwebSite.getId());
      effectiveEnquiryParamDto.setSiteCode(adwebSite.getCode());
      effectiveEnquiryParamDto.setWasteEnquirySeason(adwebEnquiry.getEmail());
      enquirySiteBlackEmailService.addBlackEmailByContent(effectiveEnquiryParamDto);
      // 清除map中的数据
      adwebRedisUtil.del(oneDayKey);
      return true;
    }
    return false;
  }

  /** 当同步询盘拉取时，处理同个ip的询盘达到一定数量后置为黑名单 如果是公共黑名单 */
  private boolean dealShortSameIp(
      AdwebEnquiry adwebEnquiry,
      List<String> blackIpList,
      AdwebSite adwebSite,
      List<String> siteBlackIpList) {
    Integer tenMin = null;
    String tenMinKey = SiteIpTenMinKey + adwebSite.getId() + "::" + adwebEnquiry.getFromIp();
    String oneDayKey = SiteIpOneDayKey + adwebSite.getId() + "::" + adwebEnquiry.getFromIp();
    if (adwebRedisUtil.hasKey(tenMinKey)) {
      tenMin = (Integer) adwebRedisUtil.get(tenMinKey) + 1;
    } else {
      tenMin = 1;
    }
    adwebRedisUtil.set(tenMinKey, tenMin, 10 * 60);
    if (tenMin > ipTenMinNum) {
      adwebEnquiry.setWasteEnquiry(1);
      adwebEnquiry.setUserEffective(0);
      adwebEnquiry.setWasteEnquiryType("ip");
      adwebEnquiry.setEffectiveReason("系统操作-通过十分钟内重复IP检测到垃圾询盘-" + adwebEnquiry.getFromIp());
      log.info(
          "询盘id是:{}, 站点code是:{}, 通过——十分钟内—重复IP—检测到垃圾询盘",
          adwebEnquiry.getId(),
          adwebEnquiry.getSiteCode());
      if (adwebSiteBlackIpService.count(
              new LambdaQueryWrapper<AdwebSiteBlackIp>()
                  .eq(AdwebSiteBlackIp::getIp, adwebEnquiry.getFromIp())
                  .ne(AdwebSiteBlackIp::getStatus, 0)
                  .eq(AdwebSiteBlackIp::getSiteId, adwebSite.getId())
                  .eq(AdwebSiteBlackIp::getBlackOrWhite, 0))
          == 0) {
        siteBlackIpList.add(adwebEnquiry.getFromIp());
        blackIpList.add(adwebEnquiry.getFromIp());
        log.info(
            "询盘id是:{}, 站点code是:{}, 检测十分钟内重复，已标记ip, ip是:{}",
            adwebEnquiry.getId(),
            adwebEnquiry.getSiteCode(),
            adwebEnquiry.getFromIp());
      }
      // 存储到数据库中
      EffectiveEnquiryParamDto effectiveEnquiryParamDto = new EffectiveEnquiryParamDto();
      effectiveEnquiryParamDto.setWasteEnquirySeason(adwebEnquiry.getFromIp());
      effectiveEnquiryParamDto.setSiteId(adwebSite.getId());
      effectiveEnquiryParamDto.setSiteCode(adwebSite.getCode());
      adwebSiteBlackIpService.addBlackIpByContent(effectiveEnquiryParamDto);
      // 清除map中的数据
      adwebRedisUtil.del(tenMinKey);
      adwebRedisUtil.del(oneDayKey);
      return true;
    }

    Integer oneDay = null;
    if (adwebRedisUtil.hasKey(oneDayKey)) {
      oneDay = (Integer) adwebRedisUtil.get(oneDayKey) + 1;
    } else {
      oneDay = 1;
    }
    adwebRedisUtil.set(oneDayKey, oneDay, 24 * 60 * 60);
    if (oneDay > ipOneDayNum) {
      adwebEnquiry.setWasteEnquiry(1);
      adwebEnquiry.setUserEffective(0);
      adwebEnquiry.setWasteEnquiryType("ip");
      adwebEnquiry.setEffectiveReason("系统操作-通过一天内重复IP检测到垃圾询盘-" + adwebEnquiry.getFromIp());
      log.info(
          "询盘id是:{}, 站点code是:{}, 通过——一天内—重复IP—检测到垃圾询盘",
          adwebEnquiry.getId(),
          adwebEnquiry.getSiteCode());
      if (adwebSiteBlackIpService.count(
              new LambdaQueryWrapper<AdwebSiteBlackIp>()
                  .eq(AdwebSiteBlackIp::getIp, adwebEnquiry.getFromIp())
                  .ne(AdwebSiteBlackIp::getStatus, 0)
                  .eq(AdwebSiteBlackIp::getSiteId, adwebSite.getId())
                  .eq(AdwebSiteBlackIp::getBlackOrWhite, 0))
          == 0) {
        siteBlackIpList.add(adwebEnquiry.getFromIp());
        blackIpList.add(adwebEnquiry.getFromIp());
        log.info(
            "询盘id是:{}, 站点code是:{}, 检测一天内重复，已标记ip, ip是:{}",
            adwebEnquiry.getId(),
            adwebEnquiry.getSiteCode(),
            adwebEnquiry.getFromIp());
      }
      // 存储到数据库中
      EffectiveEnquiryParamDto effectiveEnquiryParamDto = new EffectiveEnquiryParamDto();
      effectiveEnquiryParamDto.setWasteEnquirySeason(adwebEnquiry.getFromIp());
      effectiveEnquiryParamDto.setSiteId(adwebSite.getId());
      effectiveEnquiryParamDto.setSiteCode(adwebSite.getCode());
      adwebSiteBlackIpService.addBlackIpByContent(effectiveEnquiryParamDto);
      // 清除map中的数据
      adwebRedisUtil.del(oneDayKey);
      return true;
    }
    return false;
  }

  /**
   * 判断是否是在黑名单中
   *
   * @param adwebEnquiryList 询盘列表
   */
  private void judgeBlacklist(List<AdwebEnquiry> adwebEnquiryList) {
    for (AdwebEnquiry enquiry : adwebEnquiryList) {
      boolean isBlackEnquiry = adwebEnquiryBlacklistService.isBlackEnquiry(enquiry);
      if (isBlackEnquiry) {
        enquiry.setStatus(0);
        enquiry.setPushStatus(2);
      }
    }
  }

  @Override
  public void returnSalesperson(String siteCode, String inquiryId, String sales) {
    log.info("返回跟进人名称开始");
    // 获取站点
    LambdaQueryWrapper<AdwebSite> queryWrapper = new LambdaQueryWrapper<>();
    queryWrapper.eq(AdwebSite::getCode, siteCode);
    AdwebSite adwebSite = adwebSiteService.getOne(queryWrapper);

    RestTemplate restTemplate = new RestTemplate();
    String url = adwebSite.getDomain() + "/wp-json/inquiry/v1/add-sales";
    log.info("请求地址{}", url);
    // 如果需要传递参数
    HttpHeaders headers = new HttpHeaders();
    headers.set("Content-Type", "application/json");
    // 构建请求体，将 recordId 和 name 作为参数传递
    String requestBody = "{ \"inquiryId\": \"" + inquiryId + "\", \"sales\": \"" + sales + "\" }";
    log.info("请求参数{}", requestBody);
    HttpEntity<String> entity = new HttpEntity<>(requestBody, headers);

    // 发送请求
    restTemplate.exchange(url, HttpMethod.POST, entity, String.class);
    log.info("返回跟进人名称结束");
  }

  /** 根据地区更新询盘跟进人 */
  @Override
  public void updatePrincipalUidByRegion(Integer siteId) {
    List<Integer> siteIds = adwebSiteService.getAllSiteIdByParentId(siteId);
    adwebEnquiryMapper.updatePrincipalUidByRegion(siteId, siteIds);
  }

  /**
   * 更改询盘用户有效性
   *
   * @return
   */
  @Override
  public boolean updateUserEffective(EffectiveEnquiryParamDto effectiveEnquiryParamDto) {
    String id = effectiveEnquiryParamDto.getId();
    Integer userEffective = effectiveEnquiryParamDto.getUserEffective();
    if (userEffective == null || StringUtil.isEmpty(id)) {
      return false;
    }

    List<String> idList = Arrays.asList(id.split(","));
    if (CollectionUtils.isEmpty(idList)) {
      return false;
    }

    LambdaUpdateWrapper<AdwebEnquiry> updateWrapper = new LambdaUpdateWrapper<>();
    updateWrapper.in(AdwebEnquiry::getId, idList);
    // 用户有效性  0：无效 1：有效   2 : 待定 取自字典表
    updateWrapper.set(AdwebEnquiry::getUserEffective, userEffective);
    // 0：不是；1：是
    if (userEffective.equals(0)) { // 垃圾询盘
      String wasteEnquiryType = effectiveEnquiryParamDto.getWasteEnquiryType();
      String wasteEnquirySeason = effectiveEnquiryParamDto.getWasteEnquirySeason();
      updateWrapper.set(AdwebEnquiry::getWasteEnquiry, 1);
      updateWrapper.set(AdwebEnquiry::getWasteEnquiryType, wasteEnquiryType);

      // 特殊处理垃圾询盘是邮箱或者ip
      if (wasteEnquiryType.equals("ip")) {
        updateWrapper.set(AdwebEnquiry::getEffectiveReason, "人工操作-ip-" + wasteEnquirySeason);
        adwebSiteBlackIpService.addBlackIpByContent(effectiveEnquiryParamDto);
      }
      if (wasteEnquiryType.equals("email")) {
        updateWrapper.set(AdwebEnquiry::getEffectiveReason, "人工操作-邮箱-" + wasteEnquirySeason);
        enquirySiteBlackEmailService.addBlackEmailByContent(effectiveEnquiryParamDto);
      }
      if (wasteEnquiryType.equals("keyword")) {
        updateWrapper.set(AdwebEnquiry::getEffectiveReason, "人工操作-关键词-" + wasteEnquirySeason);
        adwebSiteEnquiryRuleService.addBlackKeywordByContent(effectiveEnquiryParamDto);
      }
      if (wasteEnquiryType.equals("other")) {
        updateWrapper.set(AdwebEnquiry::getEffectiveReason, "人工操作-其他-" + wasteEnquirySeason);
      }
    } else { // 有效询盘
      updateWrapper.set(AdwebEnquiry::getWasteEnquiry, 0);
      updateWrapper.set(AdwebEnquiry::getEffectiveReason, "人工操作");
      updateWrapper.set(AdwebEnquiry::getWasteEnquiryType, "");
    }
    this.update(updateWrapper);

    if (userEffective.equals(0)) {
      return true;
    }

    // 有效询盘发送邮件
    try {
      List<AdwebEnquiry> enquiryList =
          this.list(new LambdaQueryWrapper<AdwebEnquiry>().in(AdwebEnquiry::getId, idList));
      if (CollectionUtils.isNotEmpty(enquiryList)) {
        AdwebSite adwebSite =
            adwebSiteService.getOne(
                new LambdaQueryWrapper<AdwebSite>()
                    .eq(AdwebSite::getCode, enquiryList.get(0).getSiteCode()));
        if (adwebSite != null) {

          Map<String, String> principalEmailMap = new HashMap<>();

          // 判断是否为产品询盘
          for (AdwebEnquiry adwebEnquiry : enquiryList) {
            if (StringUtils.isNotBlank(adwebEnquiry.getCartItems())) {
              adwebEnquiry.setExistProductEnquiry(1); // 产品询盘
            }
          }

          // 判断是否为联系我们询盘
          for (AdwebEnquiry adwebEnquiry : enquiryList) {
            List<DictModel> contactEnquiryWebsites =
                dictService.queryDictItemsByCode("send_contact_enquiry_website");
            List<String> contactEnquiryCodes =
                contactEnquiryWebsites.stream().map(DictModel::getValue).toList();
            if (contactEnquiryCodes.contains(adwebSite.getCode())) {
              if (StringUtils.isNotBlank(adwebEnquiry.getFromPage())
                  && adwebEnquiry.getFromPage().contains("contact")) {
                adwebEnquiry.setExistContactEnquiry(1);
              }
            }
          }

          // 获取子账户
          List<String> subAccountIdList =
              masterSubAccountRelationService.getSubAccountIdByMaster(
                  String.valueOf(adwebSite.getUid()));
          log.info("获取的子账户id：{}", FastJsonUtil.toJSONString(subAccountIdList));
          if (CollectionUtils.isNotEmpty(subAccountIdList)) {
            subAccountIdList.add(String.valueOf(adwebSite.getUid()));
            List<SysUser> principalEmailList =
                sysUserService.list(
                    new LambdaQueryWrapper<SysUser>()
                        .in(SysUser::getId, subAccountIdList)
                        .eq(SysUser::getDelFlag, 0)
                        .isNotNull(SysUser::getEmail)
                        .ne(SysUser::getEmail, "")
                        .select(SysUser::getId, SysUser::getEmail));
            if (CollectionUtils.isNotEmpty(principalEmailList)) {
              principalEmailMap =
                  principalEmailList.stream()
                      .collect(Collectors.toMap(SysUser::getId, SysUser::getEmail));
            }
          }

          for (AdwebEnquiry adwebEnquiry : enquiryList) {
            sendValidateEmail(adwebEnquiry, adwebSite, principalEmailMap);
          }
        }
      }
    } catch (Exception e) {
      log.error("推送发送询盘邮件的消息失败", e);
    }
    return true;
  }

  /**
   * @param siteCode
   * @param startTime
   * @param endTime
   * @return
   */
  @Override
  public List<EnquiryDTO> enquiryList(String siteCode, String startTime, String endTime) {
    List<EnquiryDTO> enquiryList = adwebEnquiryMapper.getEnquiryList(siteCode, startTime, endTime);

    if (enquiryList.isEmpty()) {
      return null;
    }

    return enquiryList;
  }
}
